@@ -18,14 +18,13 @@
 #include "mpris.h"
 #include "mpris1.h"
 #include "mpris2.h"
-#include "covers/artloader.h"
 
 namespace mpris {
 
-Mpris::Mpris(Player* player, ArtLoader* art_loader, QObject* parent)
+Mpris::Mpris(Application* app, QObject* parent)
   : QObject(parent),
-    mpris1_(new mpris::Mpris1(player, art_loader, this)),
-    mpris2_(new mpris::Mpris2(player, art_loader, mpris1_, this))
+    mpris1_(new mpris::Mpris1(app, this)),
+    mpris2_(new mpris::Mpris2(app, mpris1_, this))
 {
   connect(mpris2_, SIGNAL(RaiseMainWindow()), SIGNAL(RaiseMainWindow()));
   mpris2_->InitLibIndicate();
