@@ -25,6 +25,7 @@
 #include <QVariantMap>
 
 #include "config.h"
+#include "tagreadermessages.pb.h"
 #include "engines/engine_fwd.h"
 
 class QSqlQuery;
@@ -38,7 +39,7 @@ class QUrl;
   struct LIBMTP_track_struct;
 #endif
 
-#ifdef Q_OS_WIN32
+#if defined(Q_OS_WIN32) && defined(HAVE_SAC)
   struct IWMDMMetaData;
 #endif
 
@@ -48,35 +49,13 @@ class QUrl;
   }
 #endif
 
-namespace TagLib {
-  class FileRef;
-  class String;
-
-  namespace ID3v2 {
-    class Tag;
-  }
-}
-
 class SqlRow;
 
 
-class FileRefFactory {
- public:
-  virtual ~FileRefFactory() {}
-  virtual TagLib::FileRef* GetFileRef(const QString& filename) = 0;
-};
-
-class TagLibFileRefFactory : public FileRefFactory {
- public:
-  virtual TagLib::FileRef* GetFileRef(const QString& filename);
-};
-
-
 class Song {
  public:
   Song();
   Song(const Song& other);
-  Song(FileRefFactory* factory);
   ~Song();
 
   static const QStringList kColumns;
@@ -94,7 +73,8 @@ class Song {
 
   static QString JoinSpec(const QString& table);
 
-  // Don't change these values - they're stored in the database
+  // Don't change these values - they're stored in the database, and defined
+  // in the tag reader protobuf.
   enum FileType {
     Type_Unknown = 0,
     Type_Asf = 1,
@@ -115,18 +95,13 @@ class Song {
   static QString TextForFiletype(FileType type);
   QString TextForFiletype() const { return TextForFiletype(filetype()); }
 
-  // Helper function to load embedded cover art from a music file.  This is not
-  // actually used by the Song class, but instead it is called by
-  // AlbumCoverLoader and is here so it can lock on the taglib mutex.
-  static QImage LoadEmbeddedArt(const QString& filename);
-  // Checks if this Song can be properly initialized from it's media file.
-  // This requires the 'filename' attribute to be set first.
-  bool HasProperMediaFile() const;
+  // Sort songs alphabetically using their pretty title
+  static void SortSongsListAlphabetically(QList<Song>* songs);
 
   // Constructors
   void Init(const QString& title, const QString& artist, const QString& album, qint64 length_nanosec);
   void Init(const QString& title, const QString& artist, const QString& album, qint64 beginning, qint64 end);
-  void InitFromFile(const QString& filename, int directory_id);
+  void InitFromProtobuf(const pb::tagreader::SongMetadata& pb);
   void InitFromQuery(const SqlRow& query, bool reliable_metadata, int col = 0);
   void InitFromFilePartial(const QString& filename); // Just store the filename: incomplete but fast
 #ifdef HAVE_LIBLASTFM
@@ -145,21 +120,21 @@ class Song {
   void ToMTP(LIBMTP_track_struct* track) const;
 #endif
 
-#ifdef Q_OS_WIN32
+#if defined(Q_OS_WIN32) && defined(HAVE_SAC)
   void InitFromWmdm(IWMDMMetaData* metadata);
   void ToWmdm(IWMDMMetaData* metadata) const;
 #endif
 
-  static QString Decode(const TagLib::String& tag, const QTextCodec* codec = NULL);
   static QString Decode(const QString& tag, const QTextCodec* codec = NULL);
 
   // Save
   void BindToQuery(QSqlQuery* query) const;
   void BindToFtsQuery(QSqlQuery* query) const;
 #ifdef HAVE_LIBLASTFM
-  void ToLastFM(lastfm::Track* track) const;
+  void ToLastFM(lastfm::Track* track, bool prefer_album_artist) const;
 #endif
   void ToXesam(QVariantMap* map) const;
+  void ToProtobuf(pb::tagreader::SongMetadata* pb) const;
 
   // Simple accessors
   bool is_valid() const;
@@ -212,6 +187,8 @@ class Song {
   const QString& art_automatic() const;
   const QString& art_manual() const;
 
+  const QString& etag() const;
+
   // Returns true if this Song had it's cover manually unset by user.
   bool has_manually_unset_cover() const;
   // This method represents an explicit request to unset this song's
@@ -236,8 +213,6 @@ class Song {
 
   // Setters
   bool IsEditable() const;
-  bool Save() const;
-  QFuture<bool> BackgroundSave() const;
 
   void set_id(int id);
   void set_valid(bool v);
@@ -277,6 +252,7 @@ class Song {
   void set_score(int v);
   void set_cue_path(const QString& v);
   void set_unavailable(bool v);
+  void set_etag(const QString& etag);
 
   // Setters that should only be used by tests
   void set_url(const QUrl& v);
@@ -296,24 +272,9 @@ class Song {
 
   Song& operator=(const Song& other);
 
- private:
-  void GuessFileType(TagLib::FileRef* fileref);
-  static bool Save(const Song& song);
-
-  // Helper methods for taglib
-  static void SetTextFrame(const QString& id, const QString& value,
-                           TagLib::ID3v2::Tag* tag);
-  void ParseFMPSFrame(const QString& name, const QString& value);
-
  private:
   struct Private;
   QSharedDataPointer<Private> d;
-
-  FileRefFactory* factory_;
-
-  static TagLibFileRefFactory kDefaultFactory;
-
-  static QMutex sTaglibMutex;
 };
 Q_DECLARE_METATYPE(Song);
 
