@@ -19,6 +19,7 @@
 #define SONGLOADER_H
 
 #include <QObject>
+#include <QThreadPool>
 #include <QUrl>
 
 #include "song.h"
@@ -32,6 +33,7 @@ class CueParser;
 class LibraryBackendInterface;
 class ParserBase;
 class PlaylistParser;
+class PodcastParser;
 
 class SongLoader : public QObject {
   Q_OBJECT
@@ -63,6 +65,7 @@ public:
   // playlist and replace the partially-loaded items by the new ones, fully
   // loaded.
   void EffectiveSongsLoad();
+  void EffectiveSongLoad(Song* song);
   Result LoadAudioCD();
 
 signals:
@@ -95,7 +98,7 @@ private:
 
   // GStreamer callbacks
   static void TypeFound(GstElement* typefind, uint probability, GstCaps* caps, void* self);
-  static void DataReady(GstPad*, GstBuffer* buf, void* self);
+  static gboolean DataReady(GstPad*, GstBuffer* buf, void* self);
   static GstBusSyncReply BusCallbackSync(GstBus*, GstMessage*, gpointer);
   static gboolean BusCallback(GstBus*, GstMessage*, gpointer);
 
@@ -113,6 +116,7 @@ private:
 
   QTimer* timeout_timer_;
   PlaylistParser* playlist_parser_;
+  PodcastParser* podcast_parser_;
   CueParser* cue_parser_;
 
   // For async loads
@@ -121,10 +125,13 @@ private:
   bool success_;
   ParserBase* parser_;
   QString mime_type_;
+  bool is_podcast_;
   QByteArray buffer_;
   LibraryBackendInterface* library_;
 
   boost::shared_ptr<GstElement> pipeline_;
+
+  QThreadPool thread_pool_;
 };
 
 #endif // SONGLOADER_H
