@@ -11,11 +11,7 @@ class KittenLoader : public AlbumCoverLoader {
  public:
   KittenLoader(QObject* parent = 0);
 
-  virtual quint64 LoadImageAsync(
-      const QString& art_automatic,
-      const QString& art_manual,
-      const QString& song_filename,
-      const QImage& embedded_image);
+  virtual quint64 LoadKitten(const AlbumCoverLoaderOptions& options);
 
  private slots:
   void KittensRetrieved();
