@@ -54,12 +54,6 @@ void DeviceLister::ThreadStarted() {
 
 namespace {
 
-bool IsIpod(const QString& path) {
-  return QFile::exists(path + "/iTunes_Control") ||
-         QFile::exists(path + "/iPod_Control") ||
-         QFile::exists(path + "/iTunes/iTunes_Control");
-}
-
 #ifdef HAVE_LIBGPOD
 
 QString GetIpodColour(Itdb_IpodModel model) {
@@ -164,7 +158,7 @@ QString GetIpodModel(Itdb_IpodModel model) {
 
 }
 
-QUrl DeviceLister::MakeUrlFromLocalPath(const QString& path) {
+QUrl DeviceLister::MakeUrlFromLocalPath(const QString& path) const {
   if (IsIpod(path)) {
     QUrl ret;
     ret.setScheme("ipod");
@@ -175,6 +169,12 @@ QUrl DeviceLister::MakeUrlFromLocalPath(const QString& path) {
   return QUrl::fromLocalFile(path);
 }
 
+bool DeviceLister::IsIpod(const QString& path) const {
+  return QFile::exists(path + "/iTunes_Control") ||
+         QFile::exists(path + "/iPod_Control") ||
+         QFile::exists(path + "/iTunes/iTunes_Control");
+}
+
 QStringList DeviceLister::GuessIconForPath(const QString& path) {
   QStringList ret;
 
@@ -183,9 +183,6 @@ QStringList DeviceLister::GuessIconForPath(const QString& path) {
     Itdb_Device* device = itdb_device_new();
     itdb_device_set_mountpoint(device, path.toLocal8Bit().constData());
     const Itdb_IpodInfo* info = itdb_device_get_ipod_info(device);
-    qDebug() << info->model_number
-             << info->ipod_model
-             << GetIpodColour(info->ipod_model);
 
     QString colour = GetIpodColour(info->ipod_model);
     QString model = GetIpodModel(info->ipod_model);
