@@ -21,6 +21,9 @@
 #include "devicemanager.h"
 #include "devicestatefiltermodel.h"
 #include "filesystemdevice.h"
+#include "core/application.h"
+#include "core/concurrentrun.h"
+#include "core/database.h"
 #include "core/logging.h"
 #include "core/musicstorage.h"
 #include "core/taskmanager.h"
@@ -36,8 +39,10 @@
 #  include "macdevicelister.h"
 #endif
 #ifdef Q_OS_WIN32
-#  include "wmdmlister.h"
-#  include "wmdmdevice.h"
+#  ifdef HAVE_SAC
+#    include "wmdmlister.h"
+#    include "wmdmdevice.h"
+#  endif
 #endif
 #ifdef HAVE_LIBGPOD
 #  include "gpoddevice.h"
@@ -53,6 +58,7 @@
 #  include "mtpdevice.h"
 #endif
 
+#include <QApplication>
 #include <QDir>
 #include <QIcon>
 #include <QMessageBox>
@@ -61,6 +67,9 @@
 #include <QSortFilterProxyModel>
 #include <QUrl>
 
+#include <tr1/functional>
+using std::tr1::bind;
+
 const int DeviceManager::kDeviceIconSize = 32;
 const int DeviceManager::kDeviceIconOverlaySize = 16;
 
@@ -163,26 +172,22 @@ const DeviceManager::DeviceInfo::Backend* DeviceManager::DeviceInfo::BestBackend
 }
 
 
-DeviceManager::DeviceManager(BackgroundThread<Database>* database,
-                             TaskManager* task_manager, QObject *parent)
+DeviceManager::DeviceManager(Application* app, QObject *parent)
   : QAbstractListModel(parent),
-    database_(database),
-    task_manager_(task_manager),
+    app_(app),
     not_connected_overlay_(IconLoader::Load("edit-delete"))
 {
-  connect(task_manager_, SIGNAL(TasksChanged()), SLOT(TasksChanged()));
+  thread_pool_.setMaxThreadCount(1);
+  connect(app_->task_manager(), SIGNAL(TasksChanged()), SLOT(TasksChanged()));
 
   // Create the backend in the database thread
   backend_ = new DeviceDatabaseBackend;
-  backend_->moveToThread(database);
-  backend_->Init(database_->Worker());
+  backend_->moveToThread(app_->database()->thread());
+  backend_->Init(app_->database());
 
-  DeviceDatabaseBackend::DeviceList devices = backend_->GetAllDevices();
-  foreach (const DeviceDatabaseBackend::Device& device, devices) {
-    DeviceInfo info;
-    info.InitFromDb(device);
-    devices_ << info;
-  }
+  // This reads from the database and contends on the database mutex, which can
+  // be very slow on startup.
+  ConcurrentRun::Run<void>(&thread_pool_, bind(&DeviceManager::LoadAllDevices, this));
 
   // This proxy model only shows connected devices
   connected_devices_model_ = new DeviceStateFilterModel(this);
@@ -201,7 +206,7 @@ DeviceManager::DeviceManager(BackgroundThread<Database>* database,
 #ifdef Q_OS_DARWIN
   AddLister(new MacDeviceLister);
 #endif
-#ifdef Q_OS_WIN32
+#if defined(Q_OS_WIN32) && defined(HAVE_SAC)
   AddLister(new WmdmLister);
   AddDeviceClass<WmdmDevice>();
 #endif
@@ -234,6 +239,16 @@ DeviceManager::~DeviceManager() {
   backend_->deleteLater();
 }
 
+void DeviceManager::LoadAllDevices() {
+  Q_ASSERT(QThread::currentThread() != qApp->thread());
+  DeviceDatabaseBackend::DeviceList devices = backend_->GetAllDevices();
+  foreach (const DeviceDatabaseBackend::Device& device, devices) {
+    DeviceInfo info;
+    info.InitFromDb(device);
+    devices_ << info;
+  }
+}
+
 int DeviceManager::rowCount(const QModelIndex&) const {
   return devices_.count();
 }
@@ -585,7 +600,7 @@ boost::shared_ptr<ConnectedDevice> DeviceManager::Connect(int row) {
     QStringList url_strings;
     foreach (const QUrl& url, urls) { url_strings << url.toString(); }
 
-    emit Error(tr("This type of device is not supported: %1").arg(url_strings.join(", ")));
+    app_->AddError(tr("This type of device is not supported: %1").arg(url_strings.join(", ")));
     return ret;
   }
 
@@ -593,6 +608,7 @@ boost::shared_ptr<ConnectedDevice> DeviceManager::Connect(int row) {
   QObject* instance = meta_object.newInstance(
       Q_ARG(QUrl, device_url), Q_ARG(DeviceLister*, info.BestBackend()->lister_),
       Q_ARG(QString, info.BestBackend()->unique_id_), Q_ARG(DeviceManager*, this),
+      Q_ARG(Application*, app_),
       Q_ARG(int, info.database_id_), Q_ARG(bool, first_time));
   ret.reset(static_cast<ConnectedDevice*>(instance));
 
@@ -604,7 +620,6 @@ boost::shared_ptr<ConnectedDevice> DeviceManager::Connect(int row) {
     info.device_ = ret;
     emit dataChanged(index(row), index(row));
     connect(info.device_.get(), SIGNAL(TaskStarted(int)), SLOT(DeviceTaskStarted(int)));
-    connect(info.device_.get(), SIGNAL(Error(QString)), SIGNAL(Error(QString)));
     connect(info.device_.get(), SIGNAL(SongCountUpdated(int)), SLOT(DeviceSongCountUpdated(int)));
   }
 
@@ -704,7 +719,7 @@ void DeviceManager::DeviceTaskStarted(int id) {
 }
 
 void DeviceManager::TasksChanged() {
-  QList<TaskManager::Task> tasks = task_manager_->GetTasks();
+  QList<TaskManager::Task> tasks = app_->task_manager()->GetTasks();
   QList<QPersistentModelIndex> finished_tasks = active_tasks_.values();
 
   foreach (const TaskManager::Task& task, tasks) {
