@@ -17,12 +17,16 @@
 
 #include <limits>
 
+#include <QCoreApplication>
+
 #include "bufferconsumer.h"
 #include "config.h"
 #include "gstelementdeleter.h"
 #include "gstengine.h"
 #include "gstenginepipeline.h"
+#include "core/concurrentrun.h"
 #include "core/logging.h"
+#include "core/signalchecker.h"
 #include "core/utilities.h"
 #include "internet/internetmodel.h"
 
@@ -31,8 +35,6 @@
 #  include "internet/spotifyservice.h"
 #endif
 
-#include <QtConcurrentRun>
-
 const int GstEnginePipeline::kGstStateTimeoutNanosecs = 10000000;
 const int GstEnginePipeline::kFaderFudgeMsec = 2000;
 
@@ -60,6 +62,8 @@ GstEnginePipeline::GstEnginePipeline(GstEngine* engine)
     rg_preamp_(0.0),
     rg_compression_(true),
     buffer_duration_nanosec_(1 * kNsecPerSec),
+    buffering_(false),
+    mono_playback_(false),
     end_offset_nanosec_(-1),
     next_beginning_offset_nanosec_(-1),
     next_end_offset_nanosec_(-1),
@@ -109,6 +113,10 @@ void GstEnginePipeline::set_buffer_duration_nanosec(qint64 buffer_duration_nanos
   buffer_duration_nanosec_ = buffer_duration_nanosec;
 }
 
+void GstEnginePipeline::set_mono_playback(bool enabled) {
+  mono_playback_ = enabled;
+}
+
 bool GstEnginePipeline::ReplaceDecodeBin(GstElement* new_bin) {
   if (!new_bin) return false;
 
@@ -163,12 +171,9 @@ bool GstEnginePipeline::ReplaceDecodeBin(const QUrl& url) {
   } else {
     new_bin = engine_->CreateElement("uridecodebin");
     g_object_set(G_OBJECT(new_bin), "uri", url.toEncoded().constData(), NULL);
-    g_object_set(G_OBJECT(new_bin), "buffer-duration", buffer_duration_nanosec_, NULL);
-    g_object_set(G_OBJECT(new_bin), "download", true, NULL);
-    g_object_set(G_OBJECT(new_bin), "use-buffering", true, NULL);
-    g_signal_connect(G_OBJECT(new_bin), "drained", G_CALLBACK(SourceDrainedCallback), this);
-    g_signal_connect(G_OBJECT(new_bin), "pad-added", G_CALLBACK(NewPadCallback), this);
-    g_signal_connect(G_OBJECT(new_bin), "notify::source", G_CALLBACK(SourceSetupCallback), this);
+    CHECKED_GCONNECT(G_OBJECT(new_bin), "drained", &SourceDrainedCallback, this);
+    CHECKED_GCONNECT(G_OBJECT(new_bin), "pad-added", &NewPadCallback, this);
+    CHECKED_GCONNECT(G_OBJECT(new_bin), "notify::source", &SourceSetupCallback, this);
   }
 
   return ReplaceDecodeBin(new_bin);
@@ -227,7 +232,7 @@ bool GstEnginePipeline::Init() {
   GstElement *tee, *probe_queue, *probe_converter, *probe_sink, *audio_queue,
              *convert;
 
-  queue_            = engine_->CreateElement("queue",            audiobin_);
+  queue_            = engine_->CreateElement("queue2",           audiobin_);
   audioconvert_     = engine_->CreateElement("audioconvert",     audiobin_);
   tee               = engine_->CreateElement("tee",              audiobin_);
 
@@ -305,10 +310,19 @@ bool GstEnginePipeline::Init() {
     g_object_unref(G_OBJECT(band));
   }
 
-  // Set the buffer duration.  We set this on the queue as well as on the
+  // Set the buffer duration.  We set this on this queue instead of the
   // decode bin (in ReplaceDecodeBin()) because setting it on the decode bin
   // only affects network sources.
+  // Disable the default buffer and byte limits, so we only buffer based on
+  // time.
+  g_object_set(G_OBJECT(queue_), "max-size-buffers", 0, NULL);
+  g_object_set(G_OBJECT(queue_), "max-size-bytes", 0, NULL);
   g_object_set(G_OBJECT(queue_), "max-size-time", buffer_duration_nanosec_, NULL);
+  g_object_set(G_OBJECT(queue_), "low-percent", 1, NULL);
+
+  if (buffer_duration_nanosec_ > 0) {
+    g_object_set(G_OBJECT(queue_), "use-buffering", true, NULL);
+  }
 
   gst_element_link(queue_, audioconvert_);
   
@@ -321,6 +335,9 @@ bool GstEnginePipeline::Init() {
   GstCaps* caps32 = gst_caps_new_simple ("audio/x-raw-float",
       "width", G_TYPE_INT, 32,
       NULL);
+  if (mono_playback_) {
+    gst_caps_set_simple(caps32, "channels", G_TYPE_INT, 1, NULL);
+  }
 
   // Link the elements with special caps
   gst_element_link_filtered(probe_converter, probe_sink, caps16);
@@ -463,6 +480,10 @@ GstBusSyncReply GstEnginePipeline::BusCallbackSync(GstBus*, GstMessage* msg, gpo
       instance->StateChangedMessageReceived(msg);
       break;
 
+    case GST_MESSAGE_BUFFERING:
+      instance->BufferingMessageReceived(msg);
+      break;
+
     default:
       break;
   }
@@ -562,6 +583,33 @@ void GstEnginePipeline::StateChangedMessageReceived(GstMessage* msg) {
   }
 }
 
+void GstEnginePipeline::BufferingMessageReceived(GstMessage* msg) {
+  // Only handle buffering messages from the queue2 element in audiobin - not
+  // the one that's created automatically by uridecodebin.
+  if (GST_ELEMENT(GST_MESSAGE_SRC(msg)) != queue_) {
+    return;
+  }
+
+  int percent = 0;
+  gst_message_parse_buffering(msg, &percent);
+
+  const GstState current_state = state();
+
+  if (percent == 0 && current_state == GST_STATE_PLAYING && !buffering_) {
+    buffering_ = true;
+    emit BufferingStarted();
+
+    SetState(GST_STATE_PAUSED);
+  } else if (percent == 100 && buffering_) {
+    buffering_ = false;
+    emit BufferingFinished();
+
+    SetState(GST_STATE_PLAYING);
+  } else if (buffering_) {
+    emit BufferingProgress(percent);
+  }
+}
+
 void GstEnginePipeline::NewPadCallback(GstElement*, GstPad* pad, gpointer self) {
   GstEnginePipeline* instance = reinterpret_cast<GstEnginePipeline*>(self);
   GstPad* const audiopad = gst_element_get_pad(instance->audiobin_, "sink");
@@ -667,8 +715,11 @@ void GstEnginePipeline::SourceSetupCallback(GstURIDecodeBin* bin, GParamSpec *ps
   GstEnginePipeline* instance = reinterpret_cast<GstEnginePipeline*>(self);
   GstElement* element;
   g_object_get(bin, "source", &element, NULL);
-  if (element &&
-      g_object_class_find_property(G_OBJECT_GET_CLASS(element), "device") &&
+  if (!element) {
+    return;
+  }
+
+  if (g_object_class_find_property(G_OBJECT_GET_CLASS(element), "device") &&
       !instance->source_device().isEmpty()) {
     // Gstreamer is not able to handle device in URL (refering to Gstreamer
     // documentation, this might be added in the future). Despite that, for now
@@ -676,8 +727,7 @@ void GstEnginePipeline::SourceSetupCallback(GstURIDecodeBin* bin, GParamSpec *ps
     // here, when this callback is called.
     g_object_set(element, "device", instance->source_device().toLocal8Bit().constData(), NULL);
   }
-  if (element &&
-      g_object_class_find_property(G_OBJECT_GET_CLASS(element), "extra-headers") &&
+  if (g_object_class_find_property(G_OBJECT_GET_CLASS(element), "extra-headers") &&
       instance->url().host().contains("grooveshark")) {
     // Grooveshark streaming servers will answer with a 400 error 'Bad request'
     // if we don't specify 'Range' field in HTTP header.
@@ -688,6 +738,27 @@ void GstEnginePipeline::SourceSetupCallback(GstURIDecodeBin* bin, GParamSpec *ps
     g_object_set(element, "extra-headers", headers, NULL);
     gst_structure_free(headers);
   }
+
+  if (g_object_class_find_property(G_OBJECT_GET_CLASS(element), "extra-headers") &&
+      instance->url().host().contains("googleusercontent.com") &&
+      instance->url().hasFragment()) {
+    QByteArray authorization = QString("Bearer %1").arg(
+        instance->url().fragment()).toAscii();
+    GstStructure* headers = gst_structure_new(
+        "extra-headers",
+        "Authorization", G_TYPE_STRING,
+        authorization.constData(),
+        NULL);
+    g_object_set(element, "extra-headers", headers, NULL);
+    gst_structure_free(headers);
+  }
+
+  if (g_object_class_find_property(G_OBJECT_GET_CLASS(element), "user-agent")) {
+    QString user_agent = QString("%1 %2").arg(
+        QCoreApplication::applicationName(),
+        QCoreApplication::applicationVersion());
+    g_object_set(element, "user-agent", user_agent.toUtf8().constData(), NULL);
+  }
 }
 
 void GstEnginePipeline::TransitionToNext() {
@@ -743,7 +814,8 @@ GstState GstEnginePipeline::state() const {
 }
 
 QFuture<GstStateChangeReturn> GstEnginePipeline::SetState(GstState state) {
-  return QtConcurrent::run(&gst_element_set_state, pipeline_, state);
+  return ConcurrentRun::Run<GstStateChangeReturn, GstElement*, GstState>(
+      &set_state_threadpool_, &gst_element_set_state, pipeline_, state);
 }
 
 bool GstEnginePipeline::Seek(qint64 nanosec) {
