@@ -1,11 +1,12 @@
+#include "blobversion.h"
 #include "config.h"
 #include "internetmodel.h"
-#include "spotifyblobdownloader.h"
 #include "spotifyserver.h"
 #include "spotifyservice.h"
-#include "spotifysearchplaylisttype.h"
+#include "core/application.h"
 #include "core/database.h"
 #include "core/logging.h"
+#include "core/mergedproxymodel.h"
 #include "core/player.h"
 #include "core/taskmanager.h"
 #include "core/timeconstants.h"
@@ -15,20 +16,25 @@
 #include "playlist/playlist.h"
 #include "playlist/playlistcontainer.h"
 #include "playlist/playlistmanager.h"
-#include "spotifyblob/common/blobversion.h"
-#include "spotifyblob/common/spotifymessagehandler.h"
+#include "searchboxwidget.h"
 #include "widgets/didyoumean.h"
 #include "ui/iconloader.h"
+#include "widgets/didyoumean.h"
 
 #include <QCoreApplication>
 #include <QFile>
 #include <QFileInfo>
 #include <QMenu>
 #include <QMessageBox>
+#include <QMimeData>
 #include <QProcess>
 #include <QSettings>
 #include <QVariant>
 
+#ifdef HAVE_SPOTIFY_DOWNLOADER
+#include "spotifyblobdownloader.h"
+#endif
+
 Q_DECLARE_METATYPE(QStandardItem*);
 
 const char* SpotifyService::kServiceName = "Spotify";
@@ -36,20 +42,21 @@ const char* SpotifyService::kSettingsGroup = "Spotify";
 const char* SpotifyService::kBlobDownloadUrl = "http://spotify.clementine-player.org/";
 const int SpotifyService::kSearchDelayMsec = 400;
 
-SpotifyService::SpotifyService(InternetModel* parent)
-    : InternetService(kServiceName, parent, parent),
+SpotifyService::SpotifyService(Application* app, InternetModel* parent)
+    : InternetService(kServiceName, app, parent, parent),
       server_(NULL),
       blob_process_(NULL),
       root_(NULL),
       search_(NULL),
       starred_(NULL),
       inbox_(NULL),
+      toplist_(NULL),
       login_task_id_(0),
-      pending_search_playlist_(NULL),
       context_menu_(NULL),
+      search_box_(new SearchBoxWidget(this)),
       search_delay_(new QTimer(this)),
       login_state_(LoginState_OtherError),
-      bitrate_(spotify_pb::Bitrate320k),
+      bitrate_(pb::spotify::Bitrate320k),
       volume_normalisation_(false)
 {
   // Build the search path for the binary blob.
@@ -70,14 +77,12 @@ SpotifyService::SpotifyService(InternetModel* parent)
   qLog(Debug) << "Spotify system blob path:" << system_blob_path_;
   qLog(Debug) << "Spotify local blob path:" << local_blob_path_;
 
-  model()->player()->playlists()->RegisterSpecialPlaylistType(
-        new SpotifySearchPlaylistType(this));
-
-  model()->global_search()->AddProvider(new SpotifySearchProvider(this));
+  app_->global_search()->AddProvider(new SpotifySearchProvider(app_, this));
 
   search_delay_->setInterval(kSearchDelayMsec);
   search_delay_->setSingleShot(true);
   connect(search_delay_, SIGNAL(timeout()), SLOT(DoSearch()));
+  connect(search_box_, SIGNAL(TextChanged(QString)), SLOT(Search(QString)));
 }
 
 SpotifyService::~SpotifyService() {
@@ -118,6 +123,11 @@ void SpotifyService::LazyPopulate(QStandardItem* item) {
       server_->LoadUserPlaylist(item->data(Role_UserPlaylistIndex).toInt());
       break;
 
+    case Type_Toplist:
+      EnsureServerCreated();
+      server_->LoadToplist();
+      break;
+
     default:
       break;
   }
@@ -125,19 +135,15 @@ void SpotifyService::LazyPopulate(QStandardItem* item) {
   return;
 }
 
-QModelIndex SpotifyService::GetCurrentIndex() {
-  return QModelIndex();
-}
-
 void SpotifyService::Login(const QString& username, const QString& password) {
   Logout();
   EnsureServerCreated(username, password);
 }
 
 void SpotifyService::LoginCompleted(bool success, const QString& error,
-                                    spotify_pb::LoginResponse_Error error_code) {
+                                    pb::spotify::LoginResponse_Error error_code) {
   if (login_task_id_) {
-    model()->task_manager()->SetTaskFinished(login_task_id_);
+    app_->task_manager()->SetTaskFinished(login_task_id_);
     login_task_id_ = 0;
   }
 
@@ -146,19 +152,19 @@ void SpotifyService::LoginCompleted(bool success, const QString& error,
     QString error_copy(error);
 
     switch (error_code) {
-    case spotify_pb::LoginResponse_Error_BadUsernameOrPassword:
+    case pb::spotify::LoginResponse_Error_BadUsernameOrPassword:
       login_state_ = LoginState_BadCredentials;
       break;
 
-    case spotify_pb::LoginResponse_Error_UserBanned:
+    case pb::spotify::LoginResponse_Error_UserBanned:
       login_state_ = LoginState_Banned;
       break;
 
-    case spotify_pb::LoginResponse_Error_UserNeedsPremium:
+    case pb::spotify::LoginResponse_Error_UserNeedsPremium:
       login_state_ = LoginState_NoPremium;
       break;
 
-    case spotify_pb::LoginResponse_Error_ReloginFailed:
+    case pb::spotify::LoginResponse_Error_ReloginFailed:
       if (login_state_ == LoginState_LoggedIn) {
         // This is the first time the relogin has failed - show a message this
         // time only.
@@ -195,7 +201,7 @@ void SpotifyService::BlobProcessError(QProcess::ProcessError error) {
   blob_process_ = NULL;
 
   if (login_task_id_) {
-    model()->task_manager()->SetTaskFinished(login_task_id_);
+    app_->task_manager()->SetTaskFinished(login_task_id_);
   }
 }
 
@@ -204,8 +210,8 @@ void SpotifyService::ReloadSettings() {
   s.beginGroup(kSettingsGroup);
 
   login_state_ = LoginState(s.value("login_state", LoginState_OtherError).toInt());
-  bitrate_ = static_cast<spotify_pb::Bitrate>(
-        s.value("bitrate", spotify_pb::Bitrate320k).toInt());
+  bitrate_ = static_cast<pb::spotify::Bitrate>(
+        s.value("bitrate", pb::spotify::Bitrate320k).toInt());
   volume_normalisation_ = s.value("volume_normalisation", false).toBool();
 
   if (server_ && blob_process_) {
@@ -222,28 +228,30 @@ void SpotifyService::EnsureServerCreated(const QString& username,
   delete server_;
   server_ = new SpotifyServer(this);
 
-  connect(server_, SIGNAL(LoginCompleted(bool,QString,spotify_pb::LoginResponse_Error)),
-                   SLOT(LoginCompleted(bool,QString,spotify_pb::LoginResponse_Error)));
-  connect(server_, SIGNAL(PlaylistsUpdated(spotify_pb::Playlists)),
-          SLOT(PlaylistsUpdated(spotify_pb::Playlists)));
-  connect(server_, SIGNAL(InboxLoaded(spotify_pb::LoadPlaylistResponse)),
-          SLOT(InboxLoaded(spotify_pb::LoadPlaylistResponse)));
-  connect(server_, SIGNAL(StarredLoaded(spotify_pb::LoadPlaylistResponse)),
-          SLOT(StarredLoaded(spotify_pb::LoadPlaylistResponse)));
-  connect(server_, SIGNAL(UserPlaylistLoaded(spotify_pb::LoadPlaylistResponse)),
-          SLOT(UserPlaylistLoaded(spotify_pb::LoadPlaylistResponse)));
+  connect(server_, SIGNAL(LoginCompleted(bool,QString,pb::spotify::LoginResponse_Error)),
+                   SLOT(LoginCompleted(bool,QString,pb::spotify::LoginResponse_Error)));
+  connect(server_, SIGNAL(PlaylistsUpdated(pb::spotify::Playlists)),
+          SLOT(PlaylistsUpdated(pb::spotify::Playlists)));
+  connect(server_, SIGNAL(InboxLoaded(pb::spotify::LoadPlaylistResponse)),
+          SLOT(InboxLoaded(pb::spotify::LoadPlaylistResponse)));
+  connect(server_, SIGNAL(StarredLoaded(pb::spotify::LoadPlaylistResponse)),
+          SLOT(StarredLoaded(pb::spotify::LoadPlaylistResponse)));
+  connect(server_, SIGNAL(UserPlaylistLoaded(pb::spotify::LoadPlaylistResponse)),
+          SLOT(UserPlaylistLoaded(pb::spotify::LoadPlaylistResponse)));
   connect(server_, SIGNAL(PlaybackError(QString)),
           SIGNAL(StreamError(QString)));
-  connect(server_, SIGNAL(SearchResults(spotify_pb::SearchResponse)),
-          SLOT(SearchResults(spotify_pb::SearchResponse)));
+  connect(server_, SIGNAL(SearchResults(pb::spotify::SearchResponse)),
+          SLOT(SearchResults(pb::spotify::SearchResponse)));
   connect(server_, SIGNAL(ImageLoaded(QString,QImage)),
           SIGNAL(ImageLoaded(QString,QImage)));
-  connect(server_, SIGNAL(SyncPlaylistProgress(spotify_pb::SyncPlaylistProgress)),
-          SLOT(SyncPlaylistProgress(spotify_pb::SyncPlaylistProgress)));
+  connect(server_, SIGNAL(SyncPlaylistProgress(pb::spotify::SyncPlaylistProgress)),
+          SLOT(SyncPlaylistProgress(pb::spotify::SyncPlaylistProgress)));
+  connect(server_, SIGNAL(ToplistBrowseResults(pb::spotify::BrowseToplistResponse)),
+          SLOT(ToplistLoaded(pb::spotify::BrowseToplistResponse)));
 
   server_->Init();
 
-  login_task_id_ = model()->task_manager()->StartTask(tr("Connecting to Spotify"));
+  login_task_id_ = app_->task_manager()->StartTask(tr("Connecting to Spotify"));
 
   QString login_username = username;
   QString login_password = password;
@@ -282,10 +290,10 @@ void SpotifyService::StartBlobProcess() {
   if (blob_path.isEmpty()) {
     // If the blob still wasn't found then we'll prompt the user to download one
     if (login_task_id_) {
-      model()->task_manager()->SetTaskFinished(login_task_id_);
+      app_->task_manager()->SetTaskFinished(login_task_id_);
     }
 
-    #ifdef Q_OS_LINUX
+    #ifdef HAVE_SPOTIFY_DOWNLOADER
       if (SpotifyBlobDownloader::Prompt()) {
         InstallBlob();
       }
@@ -314,41 +322,56 @@ bool SpotifyService::IsBlobInstalled() const {
 }
 
 void SpotifyService::InstallBlob() {
+#ifdef HAVE_SPOTIFY_DOWNLOADER
   // The downloader deletes itself when it finishes
   SpotifyBlobDownloader* downloader = new SpotifyBlobDownloader(
         local_blob_version_, QFileInfo(local_blob_path_).path(), this);
   connect(downloader, SIGNAL(Finished()), SLOT(BlobDownloadFinished()));
   connect(downloader, SIGNAL(Finished()), SIGNAL(BlobStateChanged()));
   downloader->Start();
+#endif  // HAVE_SPOTIFY_DOWNLOADER
 }
 
 void SpotifyService::BlobDownloadFinished() {
   EnsureServerCreated();
 }
 
-void SpotifyService::PlaylistsUpdated(const spotify_pb::Playlists& response) {
+void SpotifyService::PlaylistsUpdated(const pb::spotify::Playlists& response) {
   if (login_task_id_) {
-    model()->task_manager()->SetTaskFinished(login_task_id_);
+    app_->task_manager()->SetTaskFinished(login_task_id_);
     login_task_id_ = 0;
   }
 
   // Create starred and inbox playlists if they're not here already
   if (!search_) {
     search_ = new QStandardItem(IconLoader::Load("edit-find"),
-                                tr("Search Spotify (opens a new tab)"));
+                                tr("Search results"));
+    search_->setToolTip(tr("Start typing something on the search box above to "
+                           "fill this search results list"));
     search_->setData(Type_SearchResults, InternetModel::Role_Type);
-    search_->setData(InternetModel::PlayBehaviour_DoubleClickAction,
+    search_->setData(InternetModel::PlayBehaviour_MultipleItems,
                              InternetModel::Role_PlayBehaviour);
 
     starred_ = new QStandardItem(QIcon(":/star-on.png"), tr("Starred"));
     starred_->setData(Type_StarredPlaylist, InternetModel::Role_Type);
     starred_->setData(true, InternetModel::Role_CanLazyLoad);
+    starred_->setData(InternetModel::PlayBehaviour_MultipleItems,
+                             InternetModel::Role_PlayBehaviour);
 
     inbox_ = new QStandardItem(IconLoader::Load("mail-message"), tr("Inbox"));
     inbox_->setData(Type_InboxPlaylist, InternetModel::Role_Type);
     inbox_->setData(true, InternetModel::Role_CanLazyLoad);
+    inbox_->setData(InternetModel::PlayBehaviour_MultipleItems,
+                             InternetModel::Role_PlayBehaviour);
+
+    toplist_ = new QStandardItem(QIcon(), tr("Top tracks"));
+    toplist_->setData(Type_Toplist, InternetModel::Role_Type);
+    toplist_->setData(true, InternetModel::Role_CanLazyLoad);
+    toplist_->setData(InternetModel::PlayBehaviour_MultipleItems,
+                             InternetModel::Role_PlayBehaviour);
 
     root_->appendRow(search_);
+    root_->appendRow(toplist_);
     root_->appendRow(starred_);
     root_->appendRow(inbox_);
   }
@@ -366,13 +389,13 @@ void SpotifyService::PlaylistsUpdated(const spotify_pb::Playlists& response) {
   playlists_.clear();
 
   for (int i=0 ; i<response.playlist_size() ; ++i) {
-    const spotify_pb::Playlists::Playlist& msg = response.playlist(i);
+    const pb::spotify::Playlists::Playlist& msg = response.playlist(i);
 
     QStandardItem* item = new QStandardItem(QStringFromStdString(msg.name()));
     item->setData(InternetModel::Type_UserPlaylist, InternetModel::Role_Type);
     item->setData(true, InternetModel::Role_CanLazyLoad);
     item->setData(msg.index(), Role_UserPlaylistIndex);
-    item->setData(InternetModel::PlayBehaviour_SingleItem, InternetModel::Role_PlayBehaviour);
+    item->setData(InternetModel::PlayBehaviour_MultipleItems, InternetModel::Role_PlayBehaviour);
 
     root_->appendRow(item);
     playlists_ << item;
@@ -382,13 +405,13 @@ void SpotifyService::PlaylistsUpdated(const spotify_pb::Playlists& response) {
   }
 }
 
-bool SpotifyService::DoPlaylistsDiffer(const spotify_pb::Playlists& response) const {
+bool SpotifyService::DoPlaylistsDiffer(const pb::spotify::Playlists& response) const {
   if (playlists_.count() != response.playlist_size()) {
     return true;
   }
 
   for (int i=0 ; i<response.playlist_size() ; ++i) {
-    const spotify_pb::Playlists::Playlist& msg = response.playlist(i);
+    const pb::spotify::Playlists::Playlist& msg = response.playlist(i);
     const QStandardItem* item = PlaylistBySpotifyIndex(msg.index());
 
     if (!item) {
@@ -403,12 +426,22 @@ bool SpotifyService::DoPlaylistsDiffer(const spotify_pb::Playlists& response) co
   return false;
 }
 
-void SpotifyService::InboxLoaded(const spotify_pb::LoadPlaylistResponse& response) {
-  FillPlaylist(inbox_, response);
+void SpotifyService::InboxLoaded(const pb::spotify::LoadPlaylistResponse& response) {
+  if (inbox_) {
+    FillPlaylist(inbox_, response);
+  }
 }
 
-void SpotifyService::StarredLoaded(const spotify_pb::LoadPlaylistResponse& response) {
-  FillPlaylist(starred_, response);
+void SpotifyService::StarredLoaded(const pb::spotify::LoadPlaylistResponse& response) {
+  if (starred_) {
+    FillPlaylist(starred_, response);
+  }
+}
+
+void SpotifyService::ToplistLoaded(const pb::spotify::BrowseToplistResponse& response) {
+  if (toplist_) {
+    FillPlaylist(toplist_, response.track());
+  }
 }
 
 QStandardItem* SpotifyService::PlaylistBySpotifyIndex(int index) const {
@@ -420,7 +453,7 @@ QStandardItem* SpotifyService::PlaylistBySpotifyIndex(int index) const {
   return NULL;
 }
 
-void SpotifyService::UserPlaylistLoaded(const spotify_pb::LoadPlaylistResponse& response) {
+void SpotifyService::UserPlaylistLoaded(const pb::spotify::LoadPlaylistResponse& response) {
   // Find a playlist with this index
   QStandardItem* item = PlaylistBySpotifyIndex(response.request().user_playlist_index());
   if (item) {
@@ -428,26 +461,28 @@ void SpotifyService::UserPlaylistLoaded(const spotify_pb::LoadPlaylistResponse&
   }
 }
 
-void SpotifyService::FillPlaylist(QStandardItem* item, const spotify_pb::LoadPlaylistResponse& response) {
-  qLog(Debug) << "Filling playlist:" << item->text();
+void SpotifyService::FillPlaylist(
+    QStandardItem* item,
+    const google::protobuf::RepeatedPtrField<pb::spotify::Track>& tracks) {
   if (item->hasChildren())
     item->removeRows(0, item->rowCount());
 
-  for (int i=0 ; i<response.track_size() ; ++i) {
+  for (int i=0 ; i < tracks.size() ; ++i) {
     Song song;
-    SongFromProtobuf(response.track(i), &song);
+    SongFromProtobuf(tracks.Get(i), &song);
 
-    QStandardItem* child = new QStandardItem(song.PrettyTitleWithArtist());
-    child->setData(Type_Track, InternetModel::Role_Type);
-    child->setData(QVariant::fromValue(song), InternetModel::Role_SongMetadata);
-    child->setData(InternetModel::PlayBehaviour_SingleItem, InternetModel::Role_PlayBehaviour);
-    child->setData(song.url(), InternetModel::Role_Url);
+    QStandardItem* child = CreateSongItem(song);
 
     item->appendRow(child);
   }
 }
 
-void SpotifyService::SongFromProtobuf(const spotify_pb::Track& track, Song* song) {
+void SpotifyService::FillPlaylist(QStandardItem* item, const pb::spotify::LoadPlaylistResponse& response) {
+  qLog(Debug) << "Filling playlist:" << item->text();
+  FillPlaylist(item, response.track());
+}
+
+void SpotifyService::SongFromProtobuf(const pb::spotify::Track& track, Song* song) {
   song->set_rating(track.starred() ? 1.0 : 0.0);
   song->set_title(QStringFromStdString(track.title()));
   song->set_album(QStringFromStdString(track.album()));
@@ -478,24 +513,36 @@ PlaylistItem::Options SpotifyService::playlistitem_options() const {
   return PlaylistItem::PauseDisabled | PlaylistItem::SeekDisabled;
 }
 
+QWidget* SpotifyService::HeaderWidget() const {
+  if (IsLoggedIn())
+    return search_box_;
+  return NULL;
+}
+
 void SpotifyService::EnsureMenuCreated() {
   if (context_menu_)
     return;
 
   context_menu_ = new QMenu;
-
-  context_menu_->addActions(GetPlaylistActions());
-  context_menu_->addSeparator();
-  context_menu_->addAction(IconLoader::Load("edit-find"), tr("Search Spotify (opens a new tab)..."), this, SLOT(OpenSearchTab()));
-  context_menu_->addSeparator();
   context_menu_->addAction(IconLoader::Load("configure"), tr("Configure Spotify..."), this, SLOT(ShowConfig()));
 
   playlist_context_menu_ = new QMenu;
+  playlist_context_menu_->addActions(GetPlaylistActions());
+  playlist_context_menu_->addSeparator();
   playlist_sync_action_ = playlist_context_menu_->addAction(
       IconLoader::Load("view-refresh"),
       tr("Make playlist available offline"),
       this,
       SLOT(SyncPlaylist()));
+  playlist_context_menu_->addSeparator();
+  playlist_context_menu_->addAction(IconLoader::Load("configure"),
+                                     tr("Configure Spotify..."),
+                                     this, SLOT(ShowConfig()));
+}
+
+void SpotifyService::ClearSearchResults() {
+  if (search_)
+    search_->removeRows(0, search_->rowCount());
 }
 
 void SpotifyService::SyncPlaylist() {
@@ -507,27 +554,34 @@ void SpotifyService::SyncPlaylist() {
       int index = item->data(Role_UserPlaylistIndex).toInt();
       server_->SyncUserPlaylist(index);
       playlist_sync_ids_[index] =
-          model()->task_manager()->StartTask(tr("Syncing Spotify playlist"));
+          app_->task_manager()->StartTask(tr("Syncing Spotify playlist"));
       break;
     }
     case Type_InboxPlaylist:
       server_->SyncInbox();
-      inbox_sync_id_ = model()->task_manager()->StartTask(tr("Syncing Spotify inbox"));
+      inbox_sync_id_ = app_->task_manager()->StartTask(tr("Syncing Spotify inbox"));
       break;
     case Type_StarredPlaylist:
       server_->SyncStarred();
-      starred_sync_id_ = model()->task_manager()->StartTask(tr("Syncing Spotify starred tracks"));
+      starred_sync_id_ = app_->task_manager()->StartTask(tr("Syncing Spotify starred tracks"));
       break;
     default:
       break;
   }
 }
 
-void SpotifyService::Search(const QString& text, Playlist* playlist, bool now) {
+void SpotifyService::Search(const QString& text, bool now) {
   EnsureServerCreated();
 
   pending_search_ = text;
-  pending_search_playlist_ = playlist;
+
+  // If there is no text (e.g. user cleared search box), we don't need to do a
+  // real query that will return nothing: we can clear the playlist now
+  if (text.isEmpty()) {
+    search_delay_->stop();
+    ClearSearchResults();
+    return;
+  }
 
   if (now) {
     search_delay_->stop();
@@ -543,7 +597,7 @@ void SpotifyService::DoSearch() {
   }
 }
 
-void SpotifyService::SearchResults(const spotify_pb::SearchResponse& response) {
+void SpotifyService::SearchResults(const pb::spotify::SearchResponse& response) {
   if (QStringFromStdString(response.request().query()) != pending_search_) {
     qLog(Debug) << "Old search result for"
                 << QStringFromStdString(response.request().query())
@@ -561,13 +615,25 @@ void SpotifyService::SearchResults(const spotify_pb::SearchResponse& response) {
 
   qLog(Debug) << "Got" << songs.count() << "results";
 
-  pending_search_playlist_->Clear();
-  pending_search_playlist_->InsertSongs(songs);
+  ClearSearchResults();
 
-  const QString did_you_mean = QStringFromStdString(response.did_you_mean());
-  if (!did_you_mean.isEmpty()) {
-    model()->player()->playlists()->playlist_container()->did_you_mean()->Show(did_you_mean);
+  // Fill results list
+  foreach(const Song& song, songs) {
+    QStandardItem* child = CreateSongItem(song);
+    search_->appendRow(child);
   }
+
+  const QString did_you_mean_suggestion = QStringFromStdString(response.did_you_mean());
+  qLog(Debug) << "Did you mean suggestion: " << did_you_mean_suggestion;
+  if (!did_you_mean_suggestion.isEmpty()) {
+    search_box_->did_you_mean()->Show(did_you_mean_suggestion);
+  } else {
+    // In case something else was previously displayed
+    search_box_->did_you_mean()->hide();
+  }
+
+  QModelIndex index = model()->merged_model()->mapFromSource(search_->index());
+  ScrollToIndex(index);
 }
 
 SpotifyServer* SpotifyService::server() const {
@@ -583,9 +649,9 @@ SpotifyServer* SpotifyService::server() const {
   return server_;
 }
 
-void SpotifyService::ShowContextMenu(const QModelIndex& index, const QPoint& global_pos) {
+void SpotifyService::ShowContextMenu(const QPoint& global_pos) {
   EnsureMenuCreated();
-  QStandardItem* item = model()->itemFromIndex(index);
+  QStandardItem* item = model()->itemFromIndex(model()->current_index());
   if (item) {
     int type = item->data(InternetModel::Role_Type).toInt();
     if (type == Type_InboxPlaylist ||
@@ -600,15 +666,11 @@ void SpotifyService::ShowContextMenu(const QModelIndex& index, const QPoint& glo
   context_menu_->popup(global_pos);
 }
 
-void SpotifyService::OpenSearchTab() {
-  model()->player()->playlists()->New(tr("Search Spotify"), SongList(),
-                                      SpotifySearchPlaylistType::kName);
+void SpotifyService::ItemDoubleClicked(QStandardItem* item) {
 }
 
-void SpotifyService::ItemDoubleClicked(QStandardItem* item) {
-  if (item == search_) {
-    OpenSearchTab();
-  }
+void SpotifyService::DropMimeData(const QMimeData* data, const QModelIndex& index) {
+  qLog(Debug) << Q_FUNC_INFO << data->urls();
 }
 
 void SpotifyService::LoadImage(const QString& id) {
@@ -617,17 +679,17 @@ void SpotifyService::LoadImage(const QString& id) {
 }
 
 void SpotifyService::SyncPlaylistProgress(
-    const spotify_pb::SyncPlaylistProgress& progress) {
+    const pb::spotify::SyncPlaylistProgress& progress) {
   qLog(Debug) << "Sync progress:" << progress.sync_progress();
   int task_id = -1;
   switch (progress.request().type()) {
-    case spotify_pb::Inbox:
+    case pb::spotify::Inbox:
       task_id = inbox_sync_id_;
       break;
-    case spotify_pb::Starred:
+    case pb::spotify::Starred:
       task_id = starred_sync_id_;
       break;
-    case spotify_pb::UserPlaylist: {
+    case pb::spotify::UserPlaylist: {
       QMap<int, int>::const_iterator it = playlist_sync_ids_.constFind(
           progress.request().user_playlist_index());
       if (it != playlist_sync_ids_.constEnd()) {
@@ -642,17 +704,17 @@ void SpotifyService::SyncPlaylistProgress(
     qLog(Warning) << "Received sync progress for unknown playlist";
     return;
   }
-  model()->task_manager()->SetTaskProgress(task_id, progress.sync_progress(), 100);
+  app_->task_manager()->SetTaskProgress(task_id, progress.sync_progress(), 100);
   if (progress.sync_progress() == 100) {
-    model()->task_manager()->SetTaskFinished(task_id);
-    if (progress.request().type() == spotify_pb::UserPlaylist) {
+    app_->task_manager()->SetTaskFinished(task_id);
+    if (progress.request().type() == pb::spotify::UserPlaylist) {
       playlist_sync_ids_.remove(task_id);
     }
   }
 }
 
 void SpotifyService::ShowConfig() {
-  emit OpenSettingsAtPage(SettingsDialog::Page_Spotify);
+  app_->OpenSettingsDialogAtPage(SettingsDialog::Page_Spotify);
 }
 
 void SpotifyService::Logout() {
