@@ -18,12 +18,12 @@
 #ifndef LIBRARY_H
 #define LIBRARY_H
 
-#include "core/backgroundthread.h"
-
+#include <QHash>
 #include <QObject>
 
 #include <boost/scoped_ptr.hpp>
 
+class Application;
 class Database;
 class LibraryBackend;
 class LibraryModel;
@@ -34,19 +34,15 @@ class Library : public QObject {
   Q_OBJECT
 
  public:
-  Library(BackgroundThread<Database>* db_thread, TaskManager* task_manager_,
-          QObject* parent);
+  Library(Application* app, QObject* parent);
+  ~Library();
 
   static const char* kSongsTable;
   static const char* kDirsTable;
   static const char* kSubdirsTable;
   static const char* kFtsTable;
 
-  // Useful for tests.  The library takes ownership.
-  void set_watcher_factory(BackgroundThreadFactory<LibraryWatcher>* factory);
-
   void Init();
-  void StartThreads();
 
   LibraryBackend* backend() const { return backend_; }
   LibraryModel* model() const { return model_; }
@@ -63,15 +59,14 @@ class Library : public QObject {
 
  private slots:
   void IncrementalScan();
-  void WatcherInitialised();
 
  private:
-  TaskManager* task_manager_;
+  Application* app_;
   LibraryBackend* backend_;
   LibraryModel* model_;
 
-  boost::scoped_ptr<BackgroundThreadFactory<LibraryWatcher> > watcher_factory_;
-  BackgroundThread<LibraryWatcher>* watcher_;
+  LibraryWatcher* watcher_;
+  QThread* watcher_thread_;
 
   // DB schema versions which should trigger a full library rescan (each of those with
   // a short reason why).
