@@ -24,52 +24,45 @@
 #endif // Q_OS_WIN32
 
 #include "config.h"
+#include "core/application.h"
 #include "core/commandlineoptions.h"
 #include "core/crashreporting.h"
-#include "core/database.h"
 #include "core/encoding.h"
+#include "core/database.h"
 #include "core/logging.h"
 #include "core/mac_startup.h"
+#include "core/metatypes.h"
 #include "core/network.h"
 #include "core/networkproxyfactory.h"
-#include "core/player.h"
 #include "core/potranslator.h"
 #include "core/song.h"
-#include "core/taskmanager.h"
 #include "core/ubuntuunityhack.h"
 #include "core/utilities.h"
-#include "covers/albumcoverfetcher.h"
 #include "covers/amazoncoverprovider.h"
-#include "covers/artloader.h"
+#include "covers/discogscoverprovider.h"
 #include "covers/coverproviders.h"
 #include "engines/enginebase.h"
-#include "globalsearch/globalsearch.h"
-#include "internet/digitallyimportedclient.h"
-#include "internet/internetmodel.h"
-#include "internet/somafmservice.h"
-#include "library/directory.h"
-#include "playlist/playlist.h"
-#include "playlist/playlistmanager.h"
 #include "smartplaylists/generator.h"
-#include "ui/equalizer.h"
 #include "ui/iconloader.h"
 #include "ui/mainwindow.h"
 #include "ui/systemtrayicon.h"
 #include "version.h"
 #include "widgets/osd.h"
 
+#include "tagreadermessages.pb.h"
+
 #include "qtsingleapplication.h"
 #include "qtsinglecoreapplication.h"
 
 #include <QDir>
 #include <QLibraryInfo>
-#include <QNetworkCookie>
 #include <QNetworkProxyFactory>
 #include <QSslSocket>
 #include <QSqlDatabase>
 #include <QSqlQuery>
 #include <QTextCodec>
 #include <QTranslator>
+#include <QtConcurrentRun>
 #include <QtDebug>
 
 #include <glib-object.h>
@@ -81,7 +74,7 @@ using boost::scoped_ptr;
 
 #include <echonest/Config.h>
 
-#ifdef HAVE_QCA
+#ifdef HAVE_SPOTIFY_DOWNLOADER
   #include <QtCrypto>
 #endif
 
@@ -99,17 +92,16 @@ using boost::scoped_ptr;
 #ifdef HAVE_DBUS
   #include "core/mpris.h"
   #include "core/mpris2.h"
-  #include "dbus/metatypes.h"
-  #include "globalsearch/globalsearchservice.h"
   #include <QDBusArgument>
-  #include <QDBusConnection>
   #include <QImage>
 
   QDBusArgument& operator<< (QDBusArgument& arg, const QImage& image);
   const QDBusArgument& operator>> (const QDBusArgument& arg, QImage& image);
 #endif
 
-class GstEnginePipeline;
+#ifdef Q_OS_WIN32
+# include <qtsparkle/Updater>
+#endif
 
 // Load sqlite plugin on windows and mac.
 #ifdef HAVE_STATIC_SQLITE
@@ -139,10 +131,6 @@ void LoadTranslation(const QString& prefix, const QString& path,
   QTextCodec::setCodecForTr(QTextCodec::codecForLocale());
 }
 
-#ifdef HAVE_REMOTE
-#include <xrme/connection.h>
-#endif
-
 void IncreaseFDLimit() {
 #ifdef Q_OS_DARWIN
   // Bump the soft limit for the number of file descriptors from the default of 256 to
@@ -164,6 +152,82 @@ void IncreaseFDLimit() {
 #endif
 }
 
+void SetEnv(const char *key, const QString &value) {
+#ifdef Q_OS_WIN32
+  putenv(QString("%1=%2").arg(key, value).toLocal8Bit().constData());
+#else
+  setenv(key, value.toLocal8Bit().constData(), 1);
+#endif
+}
+
+// This must be done early so that the spotify blob process also picks up
+// these environment variables.
+void SetGstreamerEnvironment() {
+  QString scanner_path;
+  QString plugin_path;
+  QString registry_filename;
+
+  // On windows and mac we bundle the gstreamer plugins with clementine
+#if defined(Q_OS_DARWIN)
+  scanner_path = QCoreApplication::applicationDirPath() + "/../PlugIns/gst-plugin-scanner";
+  plugin_path = QCoreApplication::applicationDirPath() + "/../PlugIns/gstreamer";
+#elif defined(Q_OS_WIN32)
+  plugin_path = QCoreApplication::applicationDirPath() + "/gstreamer-plugins";
+#endif
+
+#if defined(Q_OS_WIN32) || defined(Q_OS_DARWIN)
+  registry_filename = Utilities::GetConfigPath(Utilities::Path_GstreamerRegistry);
+#endif
+
+  if (!scanner_path.isEmpty())
+    SetEnv("GST_PLUGIN_SCANNER", scanner_path);
+
+  if (!plugin_path.isEmpty()) {
+    SetEnv("GST_PLUGIN_PATH", plugin_path);
+    // Never load plugins from anywhere else.
+    SetEnv("GST_PLUGIN_SYSTEM_PATH", plugin_path);
+  }
+
+  if (!registry_filename.isEmpty()) {
+    SetEnv("GST_REGISTRY", registry_filename);
+  }
+
+#ifdef Q_OS_DARWIN
+  SetEnv("GIO_EXTRA_MODULES",
+      QCoreApplication::applicationDirPath() + "/../PlugIns/gio-modules");
+#endif
+}
+
+#ifdef HAVE_GIO
+# undef signals // Clashes with GIO, and not needed in this file
+# include <gio/gio.h>
+
+void ScanGIOModulePath() {
+  QString gio_module_path;
+
+#if defined(Q_OS_WIN32)
+  gio_module_path = QCoreApplication::applicationDirPath() + "/gio-modules";
+#endif
+
+  if (!gio_module_path.isEmpty()) {
+    qLog(Debug) << "Adding GIO module path:" << gio_module_path;
+    QByteArray bytes = gio_module_path.toLocal8Bit();
+    g_io_modules_scan_all_in_directory(bytes.data());
+  }
+}
+#endif
+
+void ParseAProto() {
+  const QByteArray data = QByteArray::fromHex(
+        "08001a8b010a8801b2014566696c653a2f2f2f453a2f4d7573696b2f28414c42554d2"
+        "9253230476f74616e25323050726f6a6563742532302d253230416d6269656e742532"
+        "304c6f756e67652e6d786dba012a28414c42554d2920476f74616e2050726f6a65637"
+        "4202d20416d6269656e74204c6f756e67652e6d786dc001c7a7efd104c801bad685e4"
+        "04d001eeca32");
+  pb::tagreader::Message message;
+  message.ParseFromArray(data.constData(), data.size());
+}
+
 int main(int argc, char *argv[]) {
   if (CrashReporting::SendCrashReport(argc, argv)) {
     return 0;
@@ -204,45 +268,7 @@ int main(int argc, char *argv[]) {
   g_type_init();
   g_set_application_name(QCoreApplication::applicationName().toLocal8Bit());
 
-  qRegisterMetaType<CoverSearchResult>("CoverSearchResult");
-  qRegisterMetaType<QList<CoverSearchResult> >("QList<CoverSearchResult>");
-  qRegisterMetaType<CoverSearchResults>("CoverSearchResults");
-  qRegisterMetaType<Directory>("Directory");
-  qRegisterMetaType<DirectoryList>("DirectoryList");
-  qRegisterMetaType<Subdirectory>("Subdirectory");
-  qRegisterMetaType<SubdirectoryList>("SubdirectoryList");
-  qRegisterMetaType<Song>("Song");
-  qRegisterMetaType<QList<Song> >("QList<Song>");
-  qRegisterMetaType<SongList>("SongList");
-  qRegisterMetaType<PlaylistItemPtr>("PlaylistItemPtr");
-  qRegisterMetaType<QList<PlaylistItemPtr> >("QList<PlaylistItemPtr>");
-  qRegisterMetaType<PlaylistItemList>("PlaylistItemList");
-  qRegisterMetaType<Engine::State>("Engine::State");
-  qRegisterMetaType<Engine::SimpleMetaBundle>("Engine::SimpleMetaBundle");
-  qRegisterMetaType<Equalizer::Params>("Equalizer::Params");
-  qRegisterMetaTypeStreamOperators<Equalizer::Params>("Equalizer::Params");
-  qRegisterMetaType<const char*>("const char*");
-  qRegisterMetaType<QNetworkReply*>("QNetworkReply*");
-  qRegisterMetaType<QNetworkReply**>("QNetworkReply**");
-  qRegisterMetaType<smart_playlists::GeneratorPtr>("smart_playlists::GeneratorPtr");
-  qRegisterMetaType<ColumnAlignmentMap>("ColumnAlignmentMap");
-  qRegisterMetaTypeStreamOperators<QMap<int, int> >("ColumnAlignmentMap");
-  qRegisterMetaType<QNetworkCookie>("QNetworkCookie");
-  qRegisterMetaType<QList<QNetworkCookie> >("QList<QNetworkCookie>");
-  qRegisterMetaType<SearchProvider::Result>("SearchProvider::Result");
-  qRegisterMetaType<SearchProvider::ResultList>("SearchProvider::ResultList");
-  qRegisterMetaType<DigitallyImportedClient::Channel>("DigitallyImportedClient::Channel");
-  qRegisterMetaType<SomaFMService::Stream>("SomaFMService::Stream");
-  qRegisterMetaTypeStreamOperators<DigitallyImportedClient::Channel>("DigitallyImportedClient::Channel");
-  qRegisterMetaTypeStreamOperators<SomaFMService::Stream>("SomaFMService::Stream");
-
-  qRegisterMetaType<GstBuffer*>("GstBuffer*");
-  qRegisterMetaType<GstElement*>("GstElement*");
-  qRegisterMetaType<GstEnginePipeline*>("GstEnginePipeline*");
-
-#ifdef HAVE_REMOTE
-  qRegisterMetaType<xrme::SIPInfo>("xrme::SIPInfo");
-#endif
+  RegisterMetaTypes();
 
 #ifdef HAVE_LIBLASTFM
   lastfm::ws::ApiKey = LastFMService::kApiKey;
@@ -276,25 +302,30 @@ int main(int argc, char *argv[]) {
     }
   }
 
-  // Detect technically invalid usage of non-ASCII in ID3v1 tags.
-  UniversalEncodingHandler handler;
-  TagLib::ID3v1::Tag::setStringHandler(&handler);
-
-#ifdef Q_OS_LINUX
-  // Force Clementine's menu to be shown in the Clementine window and not in
-  // the Unity global menubar thing.  See:
-  // https://bugs.launchpad.net/unity/+bug/775278
-  setenv("QT_X11_NO_NATIVE_MENUBAR", "1", true);
-#endif
-
   // Initialise logging
   logging::Init();
   logging::SetLevels(options.log_levels());
   g_log_set_default_handler(reinterpret_cast<GLogFunc>(&logging::GLog), NULL);
 
+  // Seed the random number generators.
+  time_t t = time(NULL);
+  srand(t);
+  qsrand(t);
+
   IncreaseFDLimit();
 
   QtSingleApplication a(argc, argv);
+
+  // A bug in Qt means the wheel_scroll_lines setting gets ignored and replaced
+  // with the default value of 3 in QApplicationPrivate::initialize.
+  {
+    QSettings qt_settings(QSettings::UserScope, "Trolltech");
+    qt_settings.beginGroup("Qt");
+    QApplication::setWheelScrollLines(
+          qt_settings.value("wheelScrollLines",
+                            QApplication::wheelScrollLines()).toInt());
+  }
+
 #ifdef Q_OS_DARWIN
   QCoreApplication::setLibraryPaths(
       QStringList() << QCoreApplication::applicationDirPath() + "/../PlugIns");
@@ -313,8 +344,12 @@ int main(int argc, char *argv[]) {
   QCoreApplication::setAttribute(Qt::AA_DontShowIconsInMenus, false);
 #else
   QCoreApplication::setAttribute(Qt::AA_DontShowIconsInMenus, true);
+  // Fixes focus issue with NSSearchField, see QTBUG-11401
+  QCoreApplication::setAttribute(Qt::AA_NativeWindows, true);
 #endif
 
+  SetGstreamerEnvironment();
+
   // Set the permissions on the config file on Unix - it can contain passwords
   // for internet services so it's important that other users can't read it.
   // On Windows these are stored in the registry instead.
@@ -333,7 +368,7 @@ int main(int argc, char *argv[]) {
   }
 #endif
 
-#ifdef HAVE_QCA
+#ifdef HAVE_SPOTIFY_DOWNLOADER
   QCA::Initializer qca_initializer;
 #endif
 
@@ -364,9 +399,22 @@ int main(int argc, char *argv[]) {
   LoadTranslation("clementine", a.applicationDirPath(), language);
   LoadTranslation("clementine", QDir::currentPath(), language);
 
+#ifdef Q_OS_WIN32
+  // Set the language for qtsparkle
+  qtsparkle::LoadTranslations(language);
+#endif
+
   // Icons
   IconLoader::Init();
 
+  // This is a nasty hack to ensure that everything in libprotobuf is
+  // initialised in the main thread.  It fixes issue 3265 but nobody knows why.
+  // Don't remove this unless you can reproduce the error that it fixes.
+  ParseAProto();
+  QtConcurrent::run(&ParseAProto);
+
+  Application app;
+
   Echonest::Config::instance()->setAPIKey("DFLFLJBUF4EGTXHIG");
   Echonest::Config::instance()->setNetworkAccessManager(new NetworkAccessManager);
 
@@ -374,24 +422,10 @@ int main(int argc, char *argv[]) {
   QNetworkProxyFactory::setApplicationProxyFactory(
       NetworkProxyFactory::Instance());
 
-  // Seed the random number generator
-  srand(time(NULL));
-
   // Initialize the repository of cover providers.  Last.fm registers itself
   // when its service is created.
-  CoverProviders cover_providers;
-  cover_providers.AddProvider(new AmazonCoverProvider);
-
-  // Create some key objects
-  scoped_ptr<BackgroundThread<Database> > database(
-      new BackgroundThreadImplementation<Database, Database>(NULL));
-  database->Start(true);
-  TaskManager task_manager;
-  PlaylistManager playlists(&task_manager, NULL);
-  Player player(&playlists);
-  GlobalSearch global_search;
-  InternetModel internet_model(database.get(), &task_manager, &player,
-                               &cover_providers, &global_search, NULL);
+  app.cover_providers()->AddProvider(new AmazonCoverProvider);
+  app.cover_providers()->AddProvider(new DiscogsCoverProvider);
 
 #ifdef Q_OS_LINUX
   // In 11.04 Ubuntu decided that the system tray should be reserved for certain
@@ -402,37 +436,20 @@ int main(int argc, char *argv[]) {
 
   // Create the tray icon and OSD
   scoped_ptr<SystemTrayIcon> tray_icon(SystemTrayIcon::CreateSystemTrayIcon());
-  OSD osd(tray_icon.get());
-
-  ArtLoader art_loader;
+  OSD osd(tray_icon.get(), &app);
 
 #ifdef HAVE_DBUS
-  qDBusRegisterMetaType<QImage>();
-  qDBusRegisterMetaType<TrackMetadata>();
-  qDBusRegisterMetaType<TrackIds>();
-  qDBusRegisterMetaType<QList<QByteArray> >();
-
-  mpris::Mpris mpris(&player, &art_loader);
-
-  QObject::connect(&playlists, SIGNAL(CurrentSongChanged(Song)), &art_loader, SLOT(LoadArt(Song)));
-  QObject::connect(&art_loader, SIGNAL(ThumbnailLoaded(Song, QString, QImage)),
-                   &osd, SLOT(CoverArtPathReady(Song, QString)));
-
-  GlobalSearchService global_search_service(&global_search);
+  mpris::Mpris mpris(&app);
 #endif
 
   // Window
-  MainWindow w(
-      database.get(),
-      &task_manager,
-      &playlists,
-      &internet_model,
-      &player,
-      tray_icon.get(),
-      &osd,
-      &art_loader,
-      &cover_providers,
-      &global_search);
+  MainWindow w(&app, tray_icon.get(), &osd);
+#ifdef Q_OS_DARWIN
+  mac::EnableFullScreen(w);
+#endif  // Q_OS_DARWIN
+#ifdef HAVE_GIO
+  ScanGIOModulePath();
+#endif
 #ifdef HAVE_DBUS
   QObject::connect(&mpris, SIGNAL(RaiseMainWindow()), &w, SLOT(Raise()));
 #endif
@@ -453,10 +470,7 @@ int main(int argc, char *argv[]) {
   QFile self_maps("/proc/self/maps");
   if (self_maps.open(QIODevice::ReadOnly)) {
     QByteArray data = self_maps.readAll();
-    if (data.contains("libnvidia-tls.so.285.03") ||
-        data.contains("libnvidia-tls.so.280.13") ||
-        data.contains("libnvidia-tls.so.275.28") ||
-        data.contains("libnvidia-tls.so.275.19")) {
+    if (data.contains("libnvidia-tls.so.")) {
       qLog(Warning) << "Exiting immediately to work around NVIDIA driver bug";
       _exit(ret);
     }
