@@ -25,6 +25,7 @@
 
 #include "playlistitem.h"
 #include "playlistsequence.h"
+#include "core/tagreaderclient.h"
 #include "core/song.h"
 #include "smartplaylists/generator_fwd.h"
 
@@ -43,6 +44,9 @@ namespace PlaylistUndoCommands {
   class InsertItems;
   class RemoveItems;
   class MoveItems;
+  class ReOrderItems;
+  class SortItems;
+  class ShuffleItems;
 }
 
 typedef QMap<int, Qt::Alignment> ColumnAlignmentMap;
@@ -68,6 +72,7 @@ class Playlist : public QAbstractListModel {
   friend class PlaylistUndoCommands::InsertItems;
   friend class PlaylistUndoCommands::RemoveItems;
   friend class PlaylistUndoCommands::MoveItems;
+  friend class PlaylistUndoCommands::ReOrderItems;
 
  public:
   Playlist(PlaylistBackend* backend,
@@ -109,6 +114,9 @@ class Playlist : public QAbstractListModel {
 
     Column_Comment,
 
+    Column_Source,
+    Column_Mood,
+
     ColumnCount
   };
 
@@ -166,12 +174,13 @@ class Playlist : public QAbstractListModel {
   int current_row() const;
   int last_played_row() const;
   int next_row(bool ignore_repeat_track = false) const;
-  int previous_row() const;
+  int previous_row(bool ignore_repeat_track = false) const;
 
   const QModelIndex current_index() const;
 
   bool stop_after_current() const;
   bool is_dynamic() const { return dynamic_playlist_; }
+  int dynamic_history_length() const;
 
   QString special_type() const { return special_type_; }
   void set_special_type(const QString& v) { special_type_ = v; }
@@ -179,7 +188,7 @@ class Playlist : public QAbstractListModel {
   const PlaylistItemPtr& item_at(int index) const { return items_[index]; }
   const bool has_item_at(int index) const { return index >= 0 && index < rowCount(); }
 
-  PlaylistItemPtr current_item() const { return current_item_; }
+  PlaylistItemPtr current_item() const;
 
   PlaylistItem::Options current_item_options() const;
   Song current_item_metadata() const;
@@ -213,7 +222,8 @@ class Playlist : public QAbstractListModel {
                                  const SongList& songs,             int pos = -1, bool play_now = false, bool enqueue = false);
   // Removes items with given indices from the playlist. This operation is not undoable.
   void RemoveItemsWithoutUndo   (const QList<int>& indices);
-
+  void ReshuffleIndices();
+  
   // If this playlist contains the current item, this method will apply the "valid" flag on it.
   // If the "valid" flag is false, the song will be greyed out. Otherwise the grey color will
   // be undone.
@@ -240,6 +250,9 @@ class Playlist : public QAbstractListModel {
   // Unregisters a SongInsertVetoListener object.
   void RemoveSongInsertVetoListener(SongInsertVetoListener* listener);
 
+  // Just emits the dataChanged() signal so the mood column is repainted.
+  void MoodbarUpdated(const QModelIndex& index);
+
   // QAbstractListModel
   int rowCount(const QModelIndex& = QModelIndex()) const { return items_.count(); }
   int columnCount(const QModelIndex& = QModelIndex()) const { return ColumnCount; }
@@ -267,6 +280,7 @@ class Playlist : public QAbstractListModel {
   void UpdateItems(const SongList& songs);
 
   void Clear();
+  void RemoveDuplicateSongs();
   void Shuffle();
 
   void ShuffleModeChanged(PlaylistSequence::ShuffleMode mode);
@@ -293,9 +307,8 @@ class Playlist : public QAbstractListModel {
  private:
   void SetCurrentIsPaused(bool paused);
   void UpdateScrobblePoint();
-  void ReshuffleIndices();
   int NextVirtualIndex(int i, bool ignore_repeat_track) const;
-  int PreviousVirtualIndex(int i) const;
+  int PreviousVirtualIndex(int i, bool ignore_repeat_track) const;
   bool FilterContainsVirtualIndex(int i) const;
   void TurnOnDynamicPlaylist(smart_playlists::GeneratorPtr gen);
 
@@ -314,7 +327,9 @@ class Playlist : public QAbstractListModel {
                               bool enqueue = false);
   PlaylistItemList RemoveItemsWithoutUndo(int pos, int count);
   void MoveItemsWithoutUndo(const QList<int>& source_rows, int pos);
+  void MoveItemWithoutUndo(int source, int dest);
   void MoveItemsWithoutUndo(int start, const QList<int>& dest_rows);
+  void ReOrderWithoutUndo(const PlaylistItemList& new_items);
 
   void RemoveItemsNotInQueue();
 
@@ -326,7 +341,7 @@ class Playlist : public QAbstractListModel {
   void TracksDequeued();
   void TracksEnqueued(const QModelIndex&, int begin, int end);
   void QueueLayoutChanged();
-  void SongSaveComplete();
+  void SongSaveComplete(TagReaderReply* reply, const QPersistentModelIndex& index);
   void ItemReloadComplete();
   void ItemsLoaded();
   void SongInsertVetoListenerDestroyed();
@@ -356,8 +371,6 @@ class Playlist : public QAbstractListModel {
   bool current_is_paused_;
   int current_virtual_index_;
 
-  PlaylistItemPtr current_item_;
-
   bool is_shuffled_;
 
   qint64 scrobble_point_;
