@@ -23,18 +23,21 @@
 #include "playlistparsers/playlistparser.h"
 #include "ui/iconloader.h"
 #include "widgets/didyoumean.h"
-#include "widgets/maclineedit.h"
 
 #include <QFileDialog>
 #include <QInputDialog>
+#include <QKeyEvent>
 #include <QLabel>
 #include <QMessageBox>
 #include <QSettings>
 #include <QSortFilterProxyModel>
 #include <QTimeLine>
+#include <QTimer>
 #include <QUndoStack>
 
 const char* PlaylistContainer::kSettingsGroup = "Playlist";
+const int PlaylistContainer::kFilterDelayMs = 100;
+const int PlaylistContainer::kFilterDelayPlaylistSizeThreshold = 5000;
 
 PlaylistContainer::PlaylistContainer(QWidget *parent)
   : QWidget(parent),
@@ -47,7 +50,8 @@ PlaylistContainer::PlaylistContainer(QWidget *parent)
     tab_bar_visible_(false),
     tab_bar_animation_(new QTimeLine(500, this)),
     no_matches_label_(NULL),
-    did_you_mean_(NULL)
+    did_you_mean_(NULL),
+    filter_timer_(new QTimer(this))
 {
   ui_->setupUi(this);
 
@@ -83,23 +87,17 @@ PlaylistContainer::PlaylistContainer(QWidget *parent)
   connect(ui_->tab_bar, SIGNAL(currentChanged(int)), SLOT(Save()));
   connect(ui_->tab_bar, SIGNAL(Save(int)), SLOT(SavePlaylist(int)));
 
+  // set up timer for delayed filter updates
+  filter_timer_->setSingleShot(true);
+  filter_timer_->setInterval(kFilterDelayMs);
+  connect(filter_timer_,SIGNAL(timeout()),this,SLOT(UpdateFilter()));
+
   // Replace playlist search filter with native search box.
-#ifdef Q_OS_DARWIN
-  delete ui_->filter;
-  MacLineEdit* filter = new MacLineEdit(ui_->toolbar);
-  filter->setObjectName("filter");
-  ui_->horizontalLayout->addWidget(filter);
-  connect(filter, SIGNAL(textChanged(QString)), SLOT(UpdateFilter()));
-  filter->set_hint(tr("Playlist search"));
-  filter_ = filter;
-#else
-  filter_ = ui_->filter;
-  connect(ui_->filter, SIGNAL(textChanged(QString)), SLOT(UpdateFilter()));
+  connect(ui_->filter, SIGNAL(textChanged(QString)), SLOT(MaybeUpdateFilter()));
   connect(ui_->playlist, SIGNAL(FocusOnFilterSignal(QKeyEvent*)), SLOT(FocusOnFilter(QKeyEvent*)));
   ui_->filter->installEventFilter(this);
-#endif
 
-  did_you_mean_ = new DidYouMean(filter_->widget(), this);
+  did_you_mean_ = new DidYouMean(ui_->filter, this);
   connect(did_you_mean_, SIGNAL(Accepted(QString)), SLOT(DidYouMeanAccepted(QString)));
 }
 
@@ -187,7 +185,7 @@ void PlaylistContainer::SetViewModel(Playlist* playlist) {
   emit ViewSelectionModelChanged();
 
   // Update filter
-  filter_->set_text(playlist->proxy()->filterRegExp().pattern());
+  ui_->filter->setText(playlist->proxy()->filterRegExp().pattern());
 
   // Update the no matches label
   connect(playlist_->proxy(), SIGNAL(modelReset()), SLOT(UpdateNoMatchesLabel()));
@@ -221,7 +219,7 @@ void PlaylistContainer::SetViewModel(Playlist* playlist) {
 
   // Implement special playlist behaviour
   const SpecialPlaylistType* type = manager_->GetPlaylistType(playlist->special_type());
-  filter_->set_hint(type->search_hint_text(playlist));
+  ui_->filter->setPlaceholderText(type->search_hint_text(playlist));
 }
 
 void PlaylistContainer::ActivePlaying() {
@@ -385,6 +383,17 @@ void PlaylistContainer::SetTabBarHeight(int height) {
   ui_->tab_bar->setMaximumHeight(height);
 }
 
+void PlaylistContainer::MaybeUpdateFilter() {
+  // delaying the filter update on small playlists is undesirable
+  // and an empty filter applies very quickly, too
+  if (manager_->current()->rowCount() < kFilterDelayPlaylistSizeThreshold ||
+      ui_->filter->text().isEmpty()) {
+    UpdateFilter();
+  } else {
+    filter_timer_->start();
+  }
+}
+
 void PlaylistContainer::UpdateFilter() {
   Playlist* playlist = manager_->current();
   SpecialPlaylistType* type = manager_->GetPlaylistType(playlist->special_type());
@@ -392,9 +401,9 @@ void PlaylistContainer::UpdateFilter() {
   did_you_mean()->hide();
 
   if (type->has_special_search_behaviour(playlist)) {
-    type->Search(filter_->text(), playlist);
+    type->Search(ui_->filter->text(), playlist);
   } else {
-    manager_->current()->proxy()->setFilterFixedString(filter_->text());
+    manager_->current()->proxy()->setFilterFixedString(ui_->filter->text());
     ui_->playlist->JumpToCurrentlyPlayingTrack();
   }
 
@@ -431,8 +440,12 @@ void PlaylistContainer::resizeEvent(QResizeEvent* e) {
 }
 
 void PlaylistContainer::FocusOnFilter(QKeyEvent *event) {
-  ui_->filter->setFocus(Qt::OtherFocusReason);
-  QApplication::sendEvent(ui_->filter, event);
+  ui_->filter->setFocus();
+  if (event->key() == Qt::Key_Escape) {
+    ui_->filter->clear();
+  } else {
+    ui_->filter->setText(ui_->filter->text() + event->text());
+  }
 }
 
 void PlaylistContainer::RepositionNoMatchesLabel(bool force) {
@@ -469,7 +482,7 @@ bool PlaylistContainer::eventFilter(QObject *objectWatched, QEvent *event) {
           QApplication::sendEvent(ui_->playlist, event);
           return true;
         case Qt::Key_Escape:
-          ui_->filter->LineEditInterface::clear();
+          ui_->filter->clear();
           return true;
         default:
           break;
@@ -480,7 +493,7 @@ bool PlaylistContainer::eventFilter(QObject *objectWatched, QEvent *event) {
 }
 
 void PlaylistContainer::DidYouMeanAccepted(const QString& text) {
-  filter_->set_text(text);
+  ui_->filter->setText(text);
 
   Playlist* playlist = manager_->current();
   SpecialPlaylistType* type = manager_->GetPlaylistType(playlist->special_type());
