@@ -18,6 +18,7 @@
 #include "playlistdelegates.h"
 #include "queue.h"
 #include "core/logging.h"
+#include "core/player.h"
 #include "core/utilities.h"
 #include "library/librarybackend.h"
 #include "widgets/trackslider.h"
@@ -102,13 +103,15 @@ void QueuedItemDelegate::DrawBox(
   painter->setRenderHint(QPainter::Antialiasing);
 
   // Draw the box
+  painter->translate(0.5, 0.5);
   painter->setPen(QPen(Qt::white, 1));
   painter->setBrush(gradient);
   painter->drawRoundedRect(rect, kQueueBoxCornerRadius, kQueueBoxCornerRadius);
 
   // Draw the text
   painter->setFont(smaller);
-  painter->drawText(rect, Qt::AlignCenter, text);
+  painter->drawText(rect.translated(-1, -1), Qt::AlignCenter, text);
+  painter->translate(-0.5, -0.5);
 }
 
 int QueuedItemDelegate::queue_indicator_size(const QModelIndex& index) const {
@@ -423,13 +426,73 @@ QWidget* TagCompletionItemDelegate::createEditor(
 }
 
 QString NativeSeparatorsDelegate::displayText(const QVariant& value, const QLocale&) const {
-  const QString text = value.toString();
-  if (text.contains("://")) {
-    const QUrl url = QUrl::fromEncoded(text.toAscii());
-    if (url.scheme() == "file") {
-      return QDir::toNativeSeparators(url.toLocalFile());
-    }
-    return text;
+  const QString string_value = value.toString();
+
+  QUrl url;
+  if (value.type() == QVariant::Url) {
+    url = value.toUrl();
+  } else if (string_value.contains("://")) {
+    url = QUrl::fromEncoded(string_value.toAscii());
+  } else {
+    return QDir::toNativeSeparators(string_value);
   }
-  return QDir::toNativeSeparators(text);
+
+  if (url.scheme() == "file") {
+    return QDir::toNativeSeparators(url.toLocalFile());
+  }
+  return string_value;
+}
+
+SongSourceDelegate::SongSourceDelegate(QObject* parent, Player* player)
+    : PlaylistDelegateBase(parent),
+      player_(player) {
+}
+
+QString SongSourceDelegate::displayText(const QVariant& value, const QLocale&) const {
+  return QString();
+}
+
+QPixmap SongSourceDelegate::LookupPixmap(const QUrl& url, const QSize& size) const {
+  QPixmap pixmap;
+  if (cache_.find(url.scheme(), &pixmap)) {
+    return pixmap;
+  }
+
+  QIcon icon;
+  const UrlHandler* handler = player_->HandlerForUrl(url);
+  if (handler) {
+    icon = handler->icon();
+  } else {
+    if (url.scheme() == "spotify") {
+      icon = QIcon(":icons/22x22/spotify.png");
+    } else if (url.scheme() == "file") {
+      icon = IconLoader::Load("folder-sound");
+    } else if (url.host() == "api.jamendo.com") {
+      icon = QIcon(":/providers/jamendo.png");
+    } else if (url.host() == "api.soundcloud.com") {
+      icon = QIcon(":/providers/soundcloud.png");
+    }
+  }
+  pixmap = icon.pixmap(size.height());
+  cache_.insert(url.scheme(), pixmap);
+  return pixmap;
+}
+
+void SongSourceDelegate::paint(
+    QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const {
+  // Draw the background
+  PlaylistDelegateBase::paint(painter, option, index);
+
+  QStyleOptionViewItem option_copy(option);
+  initStyleOption(&option_copy, index);
+
+  // Find the pixmap to use for this URL
+  const QUrl& url = index.data().toUrl();
+  QPixmap pixmap = LookupPixmap(url, option_copy.decorationSize);
+
+  // Draw the pixmap in the middle of the rectangle
+  QRect draw_rect(QPoint(0, 0), option_copy.decorationSize);
+  draw_rect.moveCenter(option_copy.rect.center());
+
+  painter->drawPixmap(draw_rect, pixmap);
 }
