@@ -20,11 +20,15 @@
 #include <QCoreApplication>
 #include <QDir>
 #include <QFile>
+#include <QSettings>
 #include <QThread>
 #include <QtDebug>
 
 #include <boost/bind.hpp>
 
+#include "core/logging.h"
+#include "core/signalchecker.h"
+
 using boost::shared_ptr;
 
 int Transcoder::JobFinishedEvent::sEventType = -1;
@@ -60,6 +64,8 @@ GstElement* Transcoder::CreateElement(const QString &factory_name,
         tr("Could not create the GStreamer element \"%1\" -"
            " make sure you have all the required GStreamer plugins installed")
         .arg(factory_name));
+  } else {
+    SetElementProperties(factory_name, G_OBJECT(ret));
   }
 
   return ret;
@@ -146,8 +152,6 @@ GstElement* Transcoder::CreateElementForMimeType(const QString& element_type,
   if (best.name_ == "lamemp3enc") {
     // Special case: we need to add xingmux and id3v2mux to the pipeline when
     // using lamemp3enc because it doesn't write the VBR or ID3v2 headers itself.
-    // We also turn off VBR since it isn't compatible with some music players.
-    // TODO: Should make settings like these configurable.
 
     LogLine("Adding xingmux and id3v2mux to the pipeline");
 
@@ -164,10 +168,6 @@ GstElement* Transcoder::CreateElementForMimeType(const QString& element_type,
       return NULL;
     }
 
-    // Configure the lame element
-    g_object_set(G_OBJECT(lame), "target", 1, NULL); // 1 == bitrate
-    g_object_set(G_OBJECT(lame), "cbr", true, NULL);
-
     // Link the elements together
     gst_element_link_many(lame, xing, id3v2, NULL);
 
@@ -210,6 +210,17 @@ Transcoder::Transcoder(QObject* parent)
 {
   if (JobFinishedEvent::sEventType == -1)
     JobFinishedEvent::sEventType = QEvent::registerEventType();
+
+  // Initialise some settings for the lamemp3enc element.
+  QSettings s;
+  s.beginGroup("Transcoder/lamemp3enc");
+
+  if (s.value("target").isNull()) {
+    s.setValue("target", 1); // 1 == bitrate
+  }
+  if (s.value("cbr").isNull()) {
+    s.setValue("cbr", true);
+  }
 }
 
 QList<TranscoderPreset> Transcoder::GetAllPresets() {
@@ -244,7 +255,7 @@ TranscoderPreset Transcoder::PresetForFileType(Song::FileType type) {
     case Song::Type_Wav:
       return TranscoderPreset(type, "Wav",        "wav",  QString(),        "audio/x-wav");
     default:
-      qWarning() << "Unsupported format in Transcoder::PresetForFileType:" << type;
+      qLog(Warning) << "Unsupported format in PresetForFileType:" << type;
       return TranscoderPreset();
   }
 }
@@ -329,7 +340,7 @@ void Transcoder::NewPadCallback(GstElement*, GstPad* pad, gboolean, gpointer dat
   GstPad* const audiopad = gst_element_get_pad(state->convert_element_, "sink");
 
   if (GST_PAD_IS_LINKED(audiopad)) {
-    qDebug() << "audiopad is already linked. Unlinking old pad.";
+    qLog(Debug) << "audiopad is already linked, unlinking old pad";
     gst_pad_unlink(audiopad, GST_PAD_PEER(audiopad));
   }
 
@@ -392,17 +403,16 @@ bool Transcoder::StartJob(const Job &job) {
   // Create the pipeline.
   // This should be a scoped_ptr, but scoped_ptr doesn't support custom
   // destructors.
-  state->pipeline_.reset(gst_pipeline_new("pipeline"),
-                        boost::bind(gst_object_unref, _1));
+  state->pipeline_ = gst_pipeline_new("pipeline");
   if (!state->pipeline_) return false;
 
   // Create all the elements
-  GstElement* src     = CreateElement("filesrc", state->pipeline_.get());
-  GstElement* decode  = CreateElement("decodebin2", state->pipeline_.get());
-  GstElement* convert = CreateElement("audioconvert", state->pipeline_.get());
-  GstElement* codec   = CreateElementForMimeType("Codec/Encoder/Audio", job.preset.codec_mimetype_, state->pipeline_.get());
-  GstElement* muxer   = CreateElementForMimeType("Codec/Muxer", job.preset.muxer_mimetype_, state->pipeline_.get());
-  GstElement* sink    = CreateElement("filesink", state->pipeline_.get());
+  GstElement* src     = CreateElement("filesrc", state->pipeline_);
+  GstElement* decode  = CreateElement("decodebin2", state->pipeline_);
+  GstElement* convert = CreateElement("audioconvert", state->pipeline_);
+  GstElement* codec   = CreateElementForMimeType("Codec/Encoder/Audio", job.preset.codec_mimetype_, state->pipeline_);
+  GstElement* muxer   = CreateElementForMimeType("Codec/Muxer", job.preset.muxer_mimetype_, state->pipeline_);
+  GstElement* sink    = CreateElement("filesink", state->pipeline_);
 
   if (!src || !decode || !convert || !sink)
     return false;
@@ -429,18 +439,18 @@ bool Transcoder::StartJob(const Job &job) {
     gst_element_link_many(convert, muxer, sink, NULL);
 
   // Set properties
-  g_object_set(src, "location", job.input.toLocal8Bit().constData(), NULL);
-  g_object_set(sink, "location", job.output.toLocal8Bit().constData(), NULL);
+  g_object_set(src, "location", job.input.toUtf8().constData(), NULL);
+  g_object_set(sink, "location", job.output.toUtf8().constData(), NULL);
 
   // Set callbacks
   state->convert_element_ = convert;
 
-  g_signal_connect(decode, "new-decoded-pad", G_CALLBACK(NewPadCallback), state.get());
-  gst_bus_set_sync_handler(gst_pipeline_get_bus(GST_PIPELINE(state->pipeline_.get())), BusCallbackSync, state.get());
-  state->bus_callback_id_ = gst_bus_add_watch(gst_pipeline_get_bus(GST_PIPELINE(state->pipeline_.get())), BusCallback, state.get());
+  CHECKED_GCONNECT(decode, "new-decoded-pad", &NewPadCallback, state.get());
+  gst_bus_set_sync_handler(gst_pipeline_get_bus(GST_PIPELINE(state->pipeline_)), BusCallbackSync, state.get());
+  state->bus_callback_id_ = gst_bus_add_watch(gst_pipeline_get_bus(GST_PIPELINE(state->pipeline_)), BusCallback, state.get());
 
   // Start the pipeline
-  gst_element_set_state(state->pipeline_.get(), GST_STATE_PLAYING);
+  gst_element_set_state(state->pipeline_, GST_STATE_PLAYING);
 
   // GStreamer now transcodes in another thread, so we can return now and do
   // something else.  Keep the JobState object around.  It'll post an event
@@ -450,6 +460,13 @@ bool Transcoder::StartJob(const Job &job) {
   return true;
 }
 
+Transcoder::JobState::~JobState() {
+  if (pipeline_) {
+    gst_element_set_state(pipeline_, GST_STATE_NULL);
+    gst_object_unref(pipeline_);
+  }
+}
+
 bool Transcoder::event(QEvent* e) {
   if (e->type() == JobFinishedEvent::sEventType) {
     JobFinishedEvent* finished_event = static_cast<JobFinishedEvent*>(e);
@@ -472,7 +489,7 @@ bool Transcoder::event(QEvent* e) {
     // Remove event handlers from the gstreamer pipeline so they don't get
     // called after the pipeline is shutting down
     gst_bus_set_sync_handler(gst_pipeline_get_bus(GST_PIPELINE(
-        finished_event->state_->pipeline_.get())), NULL, NULL);
+        finished_event->state_->pipeline_)), NULL, NULL);
     g_source_remove(finished_event->state_->bus_callback_id_);
 
     // Remove it from the list - this will also destroy the GStreamer pipeline
@@ -501,14 +518,14 @@ void Transcoder::Cancel() {
 
     // Remove event handlers from the gstreamer pipeline so they don't get
     // called after the pipeline is shutting down
-    gst_bus_set_sync_handler(gst_pipeline_get_bus(GST_PIPELINE(
-        state->pipeline_.get())), NULL, NULL);
+    gst_bus_set_sync_handler(gst_pipeline_get_bus(
+        GST_PIPELINE(state->pipeline_)), NULL, NULL);
     g_source_remove(state->bus_callback_id_);
 
     // Stop the pipeline
-    if (gst_element_set_state(state->pipeline_.get(), GST_STATE_NULL) == GST_STATE_CHANGE_ASYNC) {
+    if (gst_element_set_state(state->pipeline_, GST_STATE_NULL) == GST_STATE_CHANGE_ASYNC) {
       // Wait for it to finish stopping...
-      gst_element_get_state(state->pipeline_.get(), NULL, NULL, GST_CLOCK_TIME_NONE);
+      gst_element_get_state(state->pipeline_, NULL, NULL, GST_CLOCK_TIME_NONE);
     }
 
     // Remove the job, this destroys the GStreamer pipeline too
@@ -527,11 +544,40 @@ QMap<QString, float> Transcoder::GetProgress() const {
     gint64 duration = 0;
     GstFormat format = GST_FORMAT_TIME;
 
-    gst_element_query_position(state->pipeline_.get(), &format, &position);
-    gst_element_query_duration(state->pipeline_.get(), &format, &duration);
+    gst_element_query_position(state->pipeline_, &format, &position);
+    gst_element_query_duration(state->pipeline_, &format, &duration);
 
     ret[state->job_.input] = float(position) / duration;
   }
 
   return ret;
 }
+
+void Transcoder::SetElementProperties(const QString& name, GObject* object) {
+  QSettings s;
+  s.beginGroup("Transcoder/" + name);
+
+  guint properties_count = 0;
+  GParamSpec** properties = g_object_class_list_properties(
+        G_OBJECT_GET_CLASS(object), &properties_count);
+
+  for (int i=0 ; i<properties_count ; ++i) {
+    GParamSpec* property = properties[i];
+
+    const QVariant value = s.value(property->name);
+    if (value.isNull())
+      continue;
+
+    LogLine(QString("Setting %1 property: %2 = %3").arg(name, property->name, value.toString()));
+
+    switch (property->value_type) {
+      case G_TYPE_DOUBLE:  g_object_set(object, property->name, value.toDouble(), NULL); break;
+      case G_TYPE_FLOAT:   g_object_set(object, property->name, value.toFloat(), NULL);  break;
+      case G_TYPE_BOOLEAN: g_object_set(object, property->name, value.toInt(), NULL);    break;
+      case G_TYPE_INT:
+      default:             g_object_set(object, property->name, value.toInt(), NULL);    break;
+    }
+  }
+
+  g_free(properties);
+}
