@@ -19,12 +19,18 @@
 #include "albumcoversearcher.h"
 #include "iconloader.h"
 #include "ui_albumcovermanager.h"
-#include "core/albumcoverfetcher.h"
+#include "core/application.h"
+#include "core/logging.h"
+#include "core/utilities.h"
+#include "covers/albumcoverfetcher.h"
+#include "covers/albumcoverloader.h"
+#include "covers/coverproviders.h"
+#include "covers/coversearchstatisticsdialog.h"
 #include "library/librarybackend.h"
 #include "library/libraryquery.h"
 #include "library/sqlrow.h"
 #include "playlist/songmimedata.h"
-#include "widgets/maclineedit.h"
+#include "widgets/forcescrollperpixel.h"
 #include "ui/albumcoverchoicecontroller.h"
 
 #include <QActionGroup>
@@ -38,30 +44,26 @@
 #include <QMessageBox>
 #include <QPainter>
 #include <QProgressBar>
-#include <QScrollBar>
 #include <QSettings>
 #include <QShortcut>
 #include <QTimer>
 
 const char* AlbumCoverManager::kSettingsGroup = "CoverManager";
 
-AlbumCoverManager::AlbumCoverManager(LibraryBackend* backend, QWidget* parent,
+AlbumCoverManager::AlbumCoverManager(Application* app,
+                                     QWidget* parent,
                                      QNetworkAccessManager* network)
   : QMainWindow(parent),
-    constructed_(false),
     ui_(new Ui_CoverManager),
+    app_(app),
     album_cover_choice_controller_(new AlbumCoverChoiceController(this)),
-    backend_(backend),
-    cover_loader_(new BackgroundThreadImplementation<AlbumCoverLoader, AlbumCoverLoader>(this)),
-    cover_fetcher_(new AlbumCoverFetcher(this, network)),
+    cover_fetcher_(new AlbumCoverFetcher(app_->cover_providers(), this, network)),
     cover_searcher_(NULL),
     artist_icon_(IconLoader::Load("x-clementine-artist")),
     all_artists_icon_(IconLoader::Load("x-clementine-album")),
     context_menu_(new QMenu(this)),
     progress_bar_(new QProgressBar(this)),
-    jobs_(0),
-    got_covers_(0),
-    missing_covers_(0)
+    jobs_(0)
 {
   ui_->setupUi(this);
   ui_->albums->set_cover_manager(this);
@@ -73,7 +75,7 @@ AlbumCoverManager::AlbumCoverManager(LibraryBackend* backend, QWidget* parent,
   ui_->action_add_to_playlist->setIcon(IconLoader::Load("media-playback-start"));
   ui_->action_load->setIcon(IconLoader::Load("media-playback-start"));
 
-  album_cover_choice_controller_->SetLibrary(backend_);
+  album_cover_choice_controller_->SetApplication(app_);
 
   // Get a square version of nocover.png
   QImage nocover(":/nocover.png");
@@ -86,7 +88,7 @@ AlbumCoverManager::AlbumCoverManager(LibraryBackend* backend, QWidget* parent,
   p.end();
   no_cover_icon_ = QPixmap::fromImage(square_nocover);
 
-  cover_searcher_ = new AlbumCoverSearcher(no_cover_icon_, this);
+  cover_searcher_ = new AlbumCoverSearcher(no_cover_icon_, app_, this);
 
   // Set up the status bar
   statusBar()->addPermanentWidget(progress_bar_);
@@ -97,6 +99,8 @@ AlbumCoverManager::AlbumCoverManager(LibraryBackend* backend, QWidget* parent,
 
   QShortcut* close = new QShortcut(QKeySequence::Close, this);
   connect(close, SIGNAL(activated()), SLOT(close()));
+
+  ResetFetchCoversButton();
 }
 
 AlbumCoverManager::~AlbumCoverManager() {
@@ -105,6 +109,10 @@ AlbumCoverManager::~AlbumCoverManager() {
   delete ui_;
 }
 
+LibraryBackend* AlbumCoverManager::backend() const {
+  return app_->library_backend();
+}
+
 void AlbumCoverManager::Init() {
   // View menu
   QActionGroup* filter_group = new QActionGroup(this);
@@ -153,25 +161,13 @@ void AlbumCoverManager::Init() {
   connect(filter_group, SIGNAL(triggered(QAction*)), SLOT(UpdateFilter()));
   connect(ui_->view, SIGNAL(clicked()), ui_->view, SLOT(showMenu()));
   connect(ui_->fetch, SIGNAL(clicked()), SLOT(FetchAlbumCovers()));
-  connect(cover_fetcher_, SIGNAL(AlbumCoverFetched(quint64,QImage)),
-          SLOT(AlbumCoverFetched(quint64,QImage)));
+  connect(cover_fetcher_, SIGNAL(AlbumCoverFetched(quint64,QImage,CoverSearchStatistics)),
+          SLOT(AlbumCoverFetched(quint64,QImage,CoverSearchStatistics)));
   connect(ui_->action_fetch, SIGNAL(triggered()), SLOT(FetchSingleCover()));
   connect(ui_->albums, SIGNAL(doubleClicked(QModelIndex)), SLOT(AlbumDoubleClicked(QModelIndex)));
   connect(ui_->action_add_to_playlist, SIGNAL(triggered()), SLOT(AddSelectedToPlaylist()));
   connect(ui_->action_load, SIGNAL(triggered()), SLOT(LoadSelectedToPlaylist()));
 
-  #ifdef Q_OS_DARWIN
-    MacLineEdit* lineedit = new MacLineEdit(ui_->filter->parentWidget());
-    lineedit->set_hint(ui_->filter->hint());
-    delete ui_->filter;
-    ui_->horizontalLayout->insertWidget(0, lineedit);
-    filter_ = lineedit;
-    connect(lineedit, SIGNAL(textChanged(QString)), SLOT(UpdateFilter()));
-    lineedit->show();
-  #else
-    filter_ = ui_->filter;
-  #endif
-
   // Restore settings
   QSettings s;
   s.beginGroup(kSettingsGroup);
@@ -182,16 +178,13 @@ void AlbumCoverManager::Init() {
     ui_->splitter->setSizes(QList<int>() << 200 << width() - 200);
   }
 
-  cover_loader_->Start(true);
-  CoverLoaderInitialised();
-  cover_searcher_->Init(cover_fetcher_);
-  constructed_ = true;
-}
-
-void AlbumCoverManager::CoverLoaderInitialised() {
-  cover_loader_->Worker()->SetDefaultOutputImage(QImage(":nocover.png"));
-  connect(cover_loader_->Worker().get(), SIGNAL(ImageLoaded(quint64,QImage)),
+  connect(app_->album_cover_loader(),
+          SIGNAL(ImageLoaded(quint64,QImage)),
           SLOT(CoverImageLoaded(quint64,QImage)));
+
+  cover_searcher_->Init(cover_fetcher_);
+
+  new ForceScrollPerPixel(ui_->albums, this);
 }
 
 void AlbumCoverManager::showEvent(QShowEvent *) {
@@ -224,27 +217,37 @@ void AlbumCoverManager::closeEvent(QCloseEvent* e) {
 }
 
 void AlbumCoverManager::CancelRequests() {
+  app_->album_cover_loader()->CancelTasks(
+        QSet<quint64>::fromList(cover_loading_tasks_.keys()));
   cover_loading_tasks_.clear();
-  if (cover_loader_ && cover_loader_->Worker()) {
-    cover_loader_->Worker()->Clear();
-  }
 
   cover_fetching_tasks_.clear();
   cover_fetcher_->Clear();
-  ui_->fetch->setEnabled(true);
   progress_bar_->hide();
   statusBar()->clearMessage();
+  ResetFetchCoversButton();
+}
+
+static bool CompareNocase(const QString& left, const QString& right) {
+  return QString::localeAwareCompare(left, right) < 0;
+}
+
+static bool CompareAlbumNameNocase(const LibraryBackend::Album& left,
+                                   const LibraryBackend::Album& right) {
+  return CompareNocase(left.album_name, right.album_name);
 }
 
 void AlbumCoverManager::Reset() {
-  if (!backend_)
-    return;
+  ResetFetchCoversButton();
 
   ui_->artists->clear();
   new QListWidgetItem(all_artists_icon_, tr("All artists"), ui_->artists, All_Artists);
   new QListWidgetItem(artist_icon_, tr("Various artists"), ui_->artists, Various_Artists);
 
-  foreach (const QString& artist, backend_->GetAllArtistsWithAlbums()) {
+  QStringList artists(app_->library_backend()->GetAllArtistsWithAlbums());
+  qStableSort(artists.begin(), artists.end(), CompareNocase);
+
+  foreach (const QString& artist, artists) {
     if (artist.isEmpty())
       continue;
 
@@ -252,9 +255,11 @@ void AlbumCoverManager::Reset() {
   }
 }
 
+void AlbumCoverManager::ResetFetchCoversButton() {
+  ui_->fetch->setEnabled(app_->cover_providers()->HasAnyProviders());
+}
+
 void AlbumCoverManager::ArtistChanged(QListWidgetItem* current) {
-  if (!backend_ || !cover_loader_->Worker())
-    return;
   if (!current)
     return;
 
@@ -270,12 +275,16 @@ void AlbumCoverManager::ArtistChanged(QListWidgetItem* current) {
   // selected in the artist list.
   LibraryBackend::AlbumList albums;
   switch (current->type()) {
-    case Various_Artists: albums = backend_->GetCompilationAlbums(); break;
-    case Specific_Artist: albums = backend_->GetAlbumsByArtist(current->text()); break;
+    case Various_Artists: albums = app_->library_backend()->GetCompilationAlbums(); break;
+    case Specific_Artist: albums = app_->library_backend()->GetAlbumsByArtist(current->text()); break;
     case All_Artists:
-    default:              albums = backend_->GetAllAlbums(); break;
+    default:              albums = app_->library_backend()->GetAllAlbums(); break;
   }
 
+  // Sort by album name.  The list is already sorted by sqlite but it was done
+  // case sensitively.
+  qStableSort(albums.begin(), albums.end(), CompareAlbumNameNocase);
+
   foreach (const LibraryBackend::Album& info, albums) {
     // Don't show songs without an album, obviously
     if (info.album_name.isEmpty())
@@ -284,14 +293,15 @@ void AlbumCoverManager::ArtistChanged(QListWidgetItem* current) {
     QListWidgetItem* item = new QListWidgetItem(no_cover_icon_, info.album_name, ui_->albums);
     item->setData(Role_ArtistName, info.artist);
     item->setData(Role_AlbumName, info.album_name);
-    item->setData(Role_FirstFilename, info.first_filename);
+    item->setData(Role_FirstUrl, info.first_url);
     item->setData(Qt::TextAlignmentRole, QVariant(Qt::AlignTop | Qt::AlignHCenter));
     item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled);
     item->setToolTip(info.artist + " - " + info.album_name);
 
     if (!info.art_automatic.isEmpty() || !info.art_manual.isEmpty()) {
-      quint64 id = cover_loader_->Worker()->LoadImageAsync(
-          info.art_automatic, info.art_manual, info.first_filename);
+      quint64 id = app_->album_cover_loader()->LoadImageAsync(
+          cover_loader_options_,
+          info.art_automatic, info.art_manual, info.first_url.toLocalFile());
       item->setData(Role_PathAutomatic, info.art_automatic);
       item->setData(Role_PathManual, info.art_manual);
       cover_loading_tasks_[id] = item;
@@ -315,7 +325,7 @@ void AlbumCoverManager::CoverImageLoaded(quint64 id, const QImage &image) {
 }
 
 void AlbumCoverManager::UpdateFilter() {
-  const QString filter = filter_->text().toLower();
+  const QString filter = ui_->filter->text().toLower();
   const bool hide_with_covers = filter_without_covers_->isChecked();
   const bool hide_without_covers = filter_with_covers_->isChecked();
 
@@ -375,58 +385,55 @@ void AlbumCoverManager::FetchAlbumCovers() {
 
   progress_bar_->setMaximum(jobs_);
   progress_bar_->show();
+  fetch_statistics_ = CoverSearchStatistics();
   UpdateStatusText();
 }
 
-void AlbumCoverManager::AlbumCoverFetched(quint64 id, const QImage &image) {
+void AlbumCoverManager::AlbumCoverFetched(quint64 id, const QImage& image,
+                                          const CoverSearchStatistics& statistics) {
   if (!cover_fetching_tasks_.contains(id))
     return;
 
   QListWidgetItem* item = cover_fetching_tasks_.take(id);
-  if (image.isNull()) {
-    missing_covers_ ++;
-  } else {
-    got_covers_ ++;
-
+  if (!image.isNull()) {
     SaveAndSetCover(item, image);
   }
 
-  if (cover_fetching_tasks_.isEmpty())
-    ui_->fetch->setEnabled(true);
+  if (cover_fetching_tasks_.isEmpty()) {
+    ResetFetchCoversButton();
+  }
 
+  fetch_statistics_ += statistics;
   UpdateStatusText();
 }
 
 void AlbumCoverManager::UpdateStatusText() {
   QString message = tr("Got %1 covers out of %2 (%3 failed)")
-                    .arg(got_covers_).arg(jobs_).arg(missing_covers_);
+                    .arg(fetch_statistics_.chosen_images_)
+                    .arg(jobs_)
+                    .arg(fetch_statistics_.missing_images_);
+
+  if (fetch_statistics_.bytes_transferred_) {
+    message += ", " + tr("%1 transferred")
+                      .arg(Utilities::PrettySize(fetch_statistics_.bytes_transferred_));
+  }
+
   statusBar()->showMessage(message);
-  progress_bar_->setValue(got_covers_ + missing_covers_);
+  progress_bar_->setValue(fetch_statistics_.chosen_images_ +
+                          fetch_statistics_.missing_images_);
 
   if (cover_fetching_tasks_.isEmpty()) {
     QTimer::singleShot(2000, statusBar(), SLOT(clearMessage()));
     progress_bar_->hide();
 
+    CoverSearchStatisticsDialog* dialog = new CoverSearchStatisticsDialog(this);
+    dialog->setAttribute(Qt::WA_DeleteOnClose);
+    dialog->Show(fetch_statistics_);
+
     jobs_ = 0;
-    got_covers_ = 0;
-    missing_covers_ = 0;
   }
 }
 
-bool AlbumCoverManager::event(QEvent* e) {
-  if (constructed_) {
-    // I think KDE styles override these, and ScrollPerItem is really confusing
-    // when you have huge items.
-    // We seem to have to reset them to sensible values each time the contents
-    // of ui_->albums changes.
-    ui_->albums->setVerticalScrollMode(QListWidget::ScrollPerPixel);
-    ui_->albums->verticalScrollBar()->setSingleStep(20);
-  }
-
-  QMainWindow::event(e);
-  return false;
-}
-
 bool AlbumCoverManager::eventFilter(QObject *obj, QEvent *event) {
   if (obj == ui_->albums && event->type() == QEvent::ContextMenu) {
     context_menu_items_ = ui_->albums->selectedItems();
@@ -444,6 +451,7 @@ bool AlbumCoverManager::eventFilter(QObject *obj, QEvent *event) {
     album_cover_choice_controller_->cover_from_url_action()->setEnabled(context_menu_items_.size() == 1);
     album_cover_choice_controller_->show_cover_action()->setEnabled(some_with_covers && context_menu_items_.size() == 1);
     album_cover_choice_controller_->unset_cover_action()->setEnabled(some_with_covers);
+    album_cover_choice_controller_->search_for_cover_action()->setEnabled(app_->cover_providers()->HasAnyProviders());
 
     QContextMenuEvent* e = static_cast<QContextMenuEvent*>(event);
     context_menu_->popup(e->globalPos());
@@ -476,7 +484,7 @@ Song AlbumCoverManager::ItemAsSong(QListWidgetItem* item) {
   result.set_artist(item->data(Role_ArtistName).toString());
   result.set_album(item->data(Role_AlbumName).toString());
 
-  result.set_filename(item->data(Role_FirstFilename).toString());
+  result.set_url(item->data(Role_FirstUrl).toUrl());
 
   result.set_art_automatic(item->data(Role_PathAutomatic).toString());
   result.set_art_manual(item->data(Role_PathManual).toString());
@@ -511,7 +519,8 @@ void AlbumCoverManager::FetchSingleCover() {
 
 
 void AlbumCoverManager::UpdateCoverInList(QListWidgetItem* item, const QString& cover) {
-  quint64 id = cover_loader_->Worker()->LoadImageAsync(QString(), cover);
+  quint64 id = app_->album_cover_loader()->LoadImageAsync(
+        cover_loader_options_, QString(), cover);
   item->setData(Role_PathManual, cover);
   cover_loading_tasks_[id] = item;
 }
@@ -625,12 +634,12 @@ SongList AlbumCoverManager::GetSongsInAlbum(const QModelIndex& index) const {
   if (!artist.isEmpty())
     q.AddWhere("artist", artist);
 
-  if (!backend_->ExecQuery(&q))
+  if (!app_->library_backend()->ExecQuery(&q))
     return ret;
 
   while (q.Next()) {
     Song song;
-    song.InitFromQuery(q);
+    song.InitFromQuery(q, true);
     ret << song;
   }
   return ret;
@@ -650,7 +659,7 @@ SongMimeData* AlbumCoverManager::GetMimeDataForAlbums(const QModelIndexList& ind
     return NULL;
 
   SongMimeData* data = new SongMimeData;
-  data->backend = backend_;
+  data->backend = app_->library_backend();
   data->songs = songs;
   return data;
 }
@@ -682,10 +691,11 @@ void AlbumCoverManager::SaveAndSetCover(QListWidgetItem *item, const QImage &ima
   QString path = album_cover_choice_controller_->SaveCoverInCache(artist, album, image);
 
   // Save the image in the database
-  backend_->UpdateManualAlbumArtAsync(artist, album, path);
+  app_->library_backend()->UpdateManualAlbumArtAsync(artist, album, path);
 
   // Update the icon in our list
-  quint64 id = cover_loader_->Worker()->LoadImageAsync(QString(), path);
+  quint64 id = app_->album_cover_loader()->LoadImageAsync(
+        cover_loader_options_, QString(), path);
   item->setData(Role_PathManual, path);
   cover_loading_tasks_[id] = item;
 }
