@@ -43,6 +43,14 @@ public:
 protected:
   // SystemTrayIcon
   void UpdateIcon();
+  void SetPaused();
+  void SetPlaying(bool enable_play_pause = false, bool enable_ban = false,
+                  bool enable_love = false);
+  void SetStopped();
+  void LastFMButtonVisibilityChanged(bool value);
+  void LastFMButtonLoveStateChanged(bool value);
+  void LastFMButtonBanStateChanged(bool value);
+  void MuteButtonStateChanged(bool value);
 
   // QObject
   bool eventFilter(QObject *, QEvent *);
@@ -53,6 +61,12 @@ private slots:
 private:
   QSystemTrayIcon* tray_;
   QMenu* menu_;
+  QAction* action_play_pause_;
+  QAction* action_stop_;
+  QAction* action_stop_after_this_track_;
+  QAction* action_mute_;
+  QAction* action_love_;
+  QAction* action_ban_;
 
   QString pattern_;
 
