@@ -15,6 +15,7 @@
    along with Clementine.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "appearancesettingspage.h"
 #include "backgroundstreamssettingspage.h"
 #include "behavioursettingspage.h"
 #include "config.h"
@@ -25,9 +26,11 @@
 #include "notificationssettingspage.h"
 #include "mainwindow.h"
 #include "settingsdialog.h"
+#include "core/application.h"
 #include "core/backgroundstreams.h"
 #include "core/logging.h"
 #include "core/networkproxyfactory.h"
+#include "core/player.h"
 #include "engines/enginebase.h"
 #include "engines/gstengine.h"
 #include "globalsearch/globalsearchsettingspage.h"
@@ -36,6 +39,7 @@
 #include "internet/magnatunesettingspage.h"
 #include "library/librarysettingspage.h"
 #include "playlist/playlistview.h"
+#include "podcasts/podcastsettingspage.h"
 #include "songinfo/songinfosettingspage.h"
 #include "transcoder/transcodersettingspage.h"
 #include "widgets/groupediconview.h"
@@ -51,14 +55,14 @@
 # include "wiimotedev/wiimotesettingspage.h"
 #endif
 
-#ifdef HAVE_REMOTE
-# include "remote/remotesettingspage.h"
-#endif
-
 #ifdef HAVE_SPOTIFY
 # include "internet/spotifysettingspage.h"
 #endif
 
+#ifdef HAVE_GOOGLE_DRIVE
+# include "internet/googledrivesettingspage.h"
+#endif
+
 #include <QDesktopWidget>
 #include <QPainter>
 #include <QPushButton>
@@ -95,13 +99,15 @@ void SettingsItemDelegate::paint(QPainter* painter, const QStyleOptionViewItem&
 }
 
 
-SettingsDialog::SettingsDialog(BackgroundStreams* streams, QWidget* parent)
+SettingsDialog::SettingsDialog(Application* app, BackgroundStreams* streams, QWidget* parent)
   : QDialog(parent),
-    model_(NULL),
-    gst_engine_(NULL),
+    app_(app),
+    model_(app_->library_model()->directory_model()),
+    gst_engine_(qobject_cast<GstEngine*>(app_->player()->engine())),
     song_info_view_(NULL),
     streams_(streams),
-    global_search_(NULL),
+    global_search_(app_->global_search()),
+    appearance_(app_->appearance()),
     ui_(new Ui_SettingsDialog),
     loading_settings_(false)
 {
@@ -116,10 +122,6 @@ SettingsDialog::SettingsDialog(BackgroundStreams* streams, QWidget* parent)
   AddPage(Page_Proxy, new NetworkProxySettingsPage(this), general);
   AddPage(Page_Transcoding, new TranscoderSettingsPage(this), general);
 
-#ifdef HAVE_REMOTE
-  AddPage(Page_Remote, new RemoteSettingsPage(this), general);
-#endif
-
 #ifdef HAVE_WIIMOTEDEV
   AddPage(Page_Wiimotedev, new WiimoteSettingsPage(this), general);
 #endif
@@ -128,6 +130,7 @@ SettingsDialog::SettingsDialog(BackgroundStreams* streams, QWidget* parent)
   QTreeWidgetItem* iface = AddCategory(tr("User interface"));
   AddPage(Page_GlobalShortcuts, new GlobalShortcutsSettingsPage(this), iface);
   AddPage(Page_GlobalSearch, new GlobalSearchSettingsPage(this), iface);
+  AddPage(Page_Appearance, new AppearanceSettingsPage(this), iface);
   AddPage(Page_SongInformation, new SongInfoSettingsPage(this), iface);
   AddPage(Page_Notifications, new NotificationsSettingsPage(this), iface);
 
@@ -140,6 +143,10 @@ SettingsDialog::SettingsDialog(BackgroundStreams* streams, QWidget* parent)
 
   AddPage(Page_Grooveshark, new GroovesharkSettingsPage(this), providers);
 
+#ifdef HAVE_GOOGLE_DRIVE
+  AddPage(Page_GoogleDrive, new GoogleDriveSettingsPage(this), providers);
+#endif
+
 #ifdef HAVE_SPOTIFY
   AddPage(Page_Spotify, new SpotifySettingsPage(this), providers);
 #endif
@@ -147,6 +154,7 @@ SettingsDialog::SettingsDialog(BackgroundStreams* streams, QWidget* parent)
   AddPage(Page_Magnatune, new MagnatuneSettingsPage(this), providers);
   AddPage(Page_DigitallyImported, new DigitallyImportedSettingsPage(this), providers);
   AddPage(Page_BackgroundStreams, new BackgroundStreamsSettingsPage(this), providers);
+  AddPage(Page_Podcasts, new PodcastSettingsPage(this), providers);
 
   // List box
   connect(ui_->list, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)),
@@ -225,6 +233,15 @@ void SettingsDialog::accept() {
   QDialog::accept();
 }
 
+void SettingsDialog::reject() {
+  // Notify each page that user clicks on Cancel
+  foreach (const PageData& data, pages_.values()) {
+    data.page_->Cancel();
+  }
+
+  QDialog::reject();
+}
+
 void SettingsDialog::showEvent(QShowEvent* e) {
   // Load settings
   loading_settings_ = true;
