@@ -27,6 +27,7 @@
 class QScrollArea;
 class QTreeWidgetItem;
 
+class Appearance;
 class BackgroundStreams;
 class GlobalSearch;
 class GlobalShortcuts;
@@ -52,7 +53,7 @@ class SettingsDialog : public QDialog {
   Q_OBJECT
 
 public:
-  SettingsDialog(BackgroundStreams* streams, QWidget* parent = 0);
+  SettingsDialog(Application* app, BackgroundStreams* streams, QWidget* parent = 0);
   ~SettingsDialog();
 
   enum Page {
@@ -61,6 +62,7 @@ public:
     Page_SongInformation,
     Page_GlobalShortcuts,
     Page_GlobalSearch,
+    Page_Appearance,
     Page_Notifications,
     Page_Library,
     Page_Lastfm,
@@ -73,31 +75,33 @@ public:
     Page_Transcoding,
     Page_Remote,
     Page_Wiimotedev,
+    Page_Podcasts,
+    Page_GoogleDrive,
   };
 
   enum Role {
     Role_IsSeparator = Qt::UserRole
   };
 
-  void SetLibraryDirectoryModel(LibraryDirectoryModel* model) { model_ = model; }
   void SetGlobalShortcutManager(GlobalShortcuts* manager) { manager_ = manager; }
-  void SetGstEngine(const GstEngine* engine) { gst_engine_ = engine; }
   void SetSongInfoView(SongInfoView* view) { song_info_view_ = view; }
-  void SetGlobalSearch(GlobalSearch* global_search) { global_search_ = global_search; }
 
   bool is_loading_settings() const { return loading_settings_; }
 
+  Application* app() const { return app_; }
   LibraryDirectoryModel* library_directory_model() const { return model_; }
   GlobalShortcuts* global_shortcuts_manager() const { return manager_; }
   const GstEngine* gst_engine() const { return gst_engine_; }
   SongInfoView* song_info_view() const { return song_info_view_; }
   BackgroundStreams* background_streams() const { return streams_; }
   GlobalSearch* global_search() const { return global_search_; }
+  Appearance* appearance() const { return appearance_; }
 
   void OpenAtPage(Page page);
 
   // QDialog
   void accept();
+  void reject();
 
   // QWidget
   void showEvent(QShowEvent* e);
@@ -120,12 +124,14 @@ private:
   void AddPage(Page id, SettingsPage* page, QTreeWidgetItem* parent = NULL);
 
 private:
+  Application* app_;
   LibraryDirectoryModel* model_;
   GlobalShortcuts* manager_;
   const GstEngine* gst_engine_;
   SongInfoView* song_info_view_;
   BackgroundStreams* streams_;
   GlobalSearch* global_search_;
+  Appearance* appearance_;
 
   Ui_SettingsDialog* ui_;
   bool loading_settings_;
