@@ -18,6 +18,7 @@
 #include "iconloader.h"
 #include "trackselectiondialog.h"
 #include "ui_trackselectiondialog.h"
+#include "core/tagreaderclient.h"
 
 #include <QFileInfo>
 #include <QFutureWatcher>
@@ -242,8 +243,9 @@ void TrackSelectionDialog::SaveData(const QList<Data>& data) {
     copy.set_artist(new_metadata.artist());
     copy.set_album(new_metadata.album());
     copy.set_track(new_metadata.track());
+    copy.set_year(new_metadata.year());
 
-    copy.Save();
+    TagReaderClient::Instance()->SaveFileBlocking(copy.url().toLocalFile(), copy);
   }
 }
 
